# -*- coding: utf-8 -*-

import json
import socket
import sys
import time
import urllib.parse
import urllib.request
import _strptime # https://bugs.python.org/issue7980
from socket import timeout
from threading import Thread
from urllib.error import HTTPError, URLError
import xbmc
import xbmcaddon
import xbmcgui
import xbmcplugin
from .allmusic import allmusic_albumfind
from .allmusic import allmusic_albumdetails
from .discogs import discogs_albumfind
from .discogs import discogs_albummain
from .discogs import discogs_albumdetails
from .fanarttv import fanarttv_albumart
from .musicbrainz import musicbrainz_albumfind
from .musicbrainz import musicbrainz_albumdetails
from .musicbrainz import musicbrainz_albumlinks
from .musicbrainz import musicbrainz_albumart
from .nfo import nfo_geturl
from .theaudiodb import theaudiodb_albumdetails
from .wikipedia import wikipedia_albumdetails
from .utils import *

ADDONID = xbmcaddon.Addon().getAddonInfo('id')
ADDONNAME = xbmcaddon.Addon().getAddonInfo('name')
ADDONVERSION = xbmcaddon.Addon().getAddonInfo('version')


def log(txt):
    message = '%s: %s' % (ADDONID, txt)
    xbmc.log(msg=message, level=xbmc.LOGDEBUG)

def get_data(url, jsonformat, retry=True):
    try:
        if url.startswith('https://musicbrainz.org/'):
            api_timeout('musicbrainztime')
        elif url.startswith('https://api.discogs.com/'):
            api_timeout('discogstime')
        headers = {}
        headers['User-Agent'] = '%s/%s ( http://kodi.tv )' % (ADDONNAME, ADDONVERSION)
        req = urllib.request.Request(url, headers=headers)
        resp = urllib.request.urlopen(req, timeout=5)
        respdata = resp.read()
    except URLError as e:
        log('URLError: %s - %s' % (e.reason, url))
        return
    except HTTPError as e:
        log('HTTPError: %s - %s' % (e.reason, url))
        return
    except socket.timeout as e:
        log('socket: %s - %s' % (e, url))
        return
    if resp.getcode() == 503:
        log('exceeding musicbrainz api limit')
        if retry:
            xbmc.sleep(1000)
            get_data(url, jsonformat, retry=False)
        else:
            return
    elif resp.getcode() == 429:
        log('exceeding discogs api limit')
        if retry:
            xbmc.sleep(1000)
            get_data(url, jsonformat, retry=False)
        else:
            return
    if jsonformat:
        respdata = json.loads(respdata)
    return respdata

def api_timeout(scraper):
    currenttime = round(time.time() * 1000)
    previoustime = xbmcgui.Window(10000).getProperty(scraper)
    if previoustime:
        timeout = currenttime - int(previoustime)
        if timeout < 1000:
            xbmc.sleep(1000 - timeout)
    xbmcgui.Window(10000).setProperty(scraper, str(round(time.time() * 1000)))


class Scraper():
    def __init__(self, action, key, artist, album, url, nfo, settings):
        # parse path settings
        self.parse_settings(settings)
        # this is just for backward compitability with xml based scrapers https://github.com/xbmc/xbmc/pull/11632
        if action == 'resolveid':
            # return the result
            result = self.resolve_mbid(key)
            self.return_resolved(result)
        # search for artist name / album title matches
        elif action == 'find':
            # try musicbrainz first
            result = self.find_album(artist, album, 'musicbrainz')
            if result:
                self.return_search(result)
            # fallback to discogs
            else:
                result = self.find_album(artist, album, 'discogs')
                if result:
                    self.return_search(result)
        # return info id's
        elif action == 'getdetails':
            details = {}
            links = {}
            url = json.loads(url)
            artist = url.get('artist')
            album = url.get('album')
            mbalbumid = url.get('mbalbumid')
            mbreleasegroupid = url.get('mbreleasegroupid')
            dcid = url.get('dcalbumid')
            threads = []
            extrascrapers = []
            # we have musicbrainz album id
            if mbalbumid:
                # get the mbreleasegroupid, artist and album if we don't have them
                if not mbreleasegroupid:
                    result = self.get_details(mbalbumid, 'musicbrainz', details)
                    if not result:
                        scrapers = [[mbalbumid, 'musicbrainz']]
                    else:
                        mbreleasegroupid = details['musicbrainz']['mbreleasegroupid']
                        artist = details['musicbrainz']['artist_description']
                        album = details['musicbrainz']['album']
                        scrapers = [[mbreleasegroupid, 'theaudiodb'], [mbreleasegroupid, 'fanarttv'], [mbreleasegroupid, 'coverarchive']]
                else:
                    scrapers = [[mbalbumid, 'musicbrainz'], [mbreleasegroupid, 'theaudiodb'], [mbreleasegroupid, 'fanarttv'], [mbreleasegroupid, 'coverarchive']]
                # get musicbrainz links to other metadata sites
                lthread = Thread(target = self.get_links, args = (mbreleasegroupid, links))
                lthread.start()
                for item in scrapers:
                    thread = Thread(target = self.get_details, args = (item[0], item[1], details))
                    threads.append(thread)
                    thread.start()
                # wait for the musicbrainz links to return
                lthread.join()
                if 'musicbrainz' in links:
                    # scrape allmusic if we have an url provided by musicbrainz
                    if 'allmusic' in links['musicbrainz']:
                        extrascrapers.append([{'url': links['musicbrainz']['allmusic']}, 'allmusic'])
                    # only scrape allmusic by artistname and albumtitle if explicitly enabled
                    elif self.inaccurate and artist and album:
                        extrascrapers.append([{'artist': artist, 'album': album}, 'allmusic'])
                    # scrape discogs if we have an url provided by musicbrainz
                    if 'discogs' in links['musicbrainz']:
                        extrascrapers.append([{'masterurl': links['musicbrainz']['discogs']}, 'discogs'])
                    # only scrape discogs by artistname and albumtitle if explicitly enabled
                    elif self.inaccurate and artist and album:
                        extrascrapers.append([{'artist': artist, 'album': album}, 'discogs'])
                    # scrape wikipedia if we have an url provided by musicbrainz
                    if 'wikipedia' in links['musicbrainz']:
                        extrascrapers.append([links['musicbrainz']['wikipedia'], 'wikipedia'])
                    elif 'wikidata' in links['musicbrainz']:
                        extrascrapers.append([links['musicbrainz']['wikidata'], 'wikidata'])
                for item in extrascrapers:
                    thread = Thread(target = self.get_details, args = (item[0], item[1], details))
                    threads.append(thread)
                    thread.start()
            # we have a discogs id
            else:
                thread = Thread(target = self.get_details, args = ({'url': dcid}, 'discogs', details))
                threads.append(thread)
                thread.start()
            for thread in threads:
                thread.join()
            result = self.compile_results(details)
            if result:
                self.return_details(result)
        # extract the mbalbumid from the provided musicbrainz url
        elif action == 'NfoUrl':
            # check if there is a musicbrainz url in the nfo file
            mbalbumid = nfo_geturl(nfo)
            if mbalbumid:
                # return the result
                result = self.resolve_mbid(mbalbumid)
                self.return_nfourl(result)
        xbmcplugin.endOfDirectory(int(sys.argv[1]))

    def parse_settings(self, data):
        settings = json.loads(data)
        # note: path settings are taken from the db, they may not reflect the current settings.xml file
        self.review = settings['review']
        self.genre = settings['genre']
        self.lang = settings['lang']
        self.mood = settings['mood']
        self.rating = settings['rating']
        self.style = settings['style']
        self.theme = settings['theme']
        self.inaccurate = settings['inaccurate']

    def resolve_mbid(self, mbalbumid):
        item = {}
        item['artist_description'] = ''
        item['album'] = ''
        item['mbalbumid'] = mbalbumid
        item['mbreleasegroupid'] = ''
        return item

    def find_album(self, artist, album, site):
        json = True
        # musicbrainz
        if site == 'musicbrainz':
            url = MUSICBRAINZURL % (MUSICBRAINZSEARCH % (urllib.parse.quote_plus(album), urllib.parse.quote_plus(artist), urllib.parse.quote_plus(artist)))
            scraper = musicbrainz_albumfind
        # discogs
        elif site == 'discogs':
            url = DISCOGSURL % (DISCOGSSEARCH % (urllib.parse.quote_plus(album), urllib.parse.quote_plus(artist), DISCOGSKEY , DISCOGSSECRET))
            scraper = discogs_albumfind
        result = get_data(url, json)
        if not result:
            return
        albumresults = scraper(result, artist, album)
        return albumresults

    def get_links(self, param, links):
        json = True
        url = MUSICBRAINZURL % (MUSICBRAINZLINKS % param)
        result = get_data(url, json)
        if result:
            linkresults = musicbrainz_albumlinks(result)
            links['musicbrainz'] = linkresults
            return links

    def get_details(self, param, site, details):
        json = True
        # theaudiodb
        if site == 'theaudiodb':
            url = AUDIODBURL % (AUDIODBKEY, AUDIODBDETAILS % param)
            albumscraper = theaudiodb_albumdetails
        # musicbrainz
        elif site == 'musicbrainz':
            url = MUSICBRAINZURL % (MUSICBRAINZDETAILS % param)
            albumscraper = musicbrainz_albumdetails
        # fanarttv
        elif site == 'fanarttv':
            url = FANARTVURL % (param, FANARTVKEY)
            albumscraper = fanarttv_albumart
       # coverarchive
        elif site == 'coverarchive':
            url = MUSICBRAINZART % (param)
            albumscraper = musicbrainz_albumart
        # discogs
        elif site == 'discogs':
            # musicbrainz provides a link to the master release, but we need the main release
            if 'masterurl' in param:
                masterdata = get_data(DISCOGSURL % (DISCOGSMASTER % (param['masterurl'], DISCOGSKEY , DISCOGSSECRET)), True)
                if masterdata:
                    url = discogs_albummain(masterdata)
                    if url:
                        param['url'] = url
                    else:
                        return
                else:
                    return
            # search by artistname and albumtitle if we do not have an url
            if not 'url' in param:
                url = DISCOGSURL % (DISCOGSSEARCH % (urllib.parse.quote_plus(param['album']), urllib.parse.quote_plus(param['artist']), DISCOGSKEY , DISCOGSSECRET))
                albumresult = get_data(url, json)
                if albumresult:
                    albums = discogs_albumfind(albumresult, param['artist'], param['album'])
                    if albums:
                        albumresult = sorted(albums, key=lambda k: k['relevance'], reverse=True)
                        param['url'] = albumresult[0]['dcalbumid']
                    else:
                        return
                else:
                    return
            url = DISCOGSURL % (DISCOGSDETAILS % (param['url'], DISCOGSKEY, DISCOGSSECRET))
            albumscraper = discogs_albumdetails
        # wikipedia
        elif site == 'wikipedia':
            url = WIKIPEDIAURL % param
            albumscraper = wikipedia_albumdetails
        elif site == 'wikidata':
            # resolve wikidata to wikipedia url
            result = get_data(WIKIDATAURL % param, json)
            try:
                album = result['entities'][param]['sitelinks']['enwiki']['url'].rsplit('/', 1)[1]
            except:
                return
            site = 'wikipedia'
            url = WIKIPEDIAURL % album
            albumscraper = wikipedia_albumdetails
        # allmusic
        elif site == 'allmusic':
            json = False
            # search by artistname and albumtitle if we do not have an url
            if not 'url' in param:
                url = ALLMUSICURL % (ALLMUSICSEARCH % (urllib.parse.quote_plus(param['artist']), urllib.parse.quote_plus(param['album'])))
                albumresult = get_data(url, json)
                if albumresult:
                    albums = allmusic_albumfind(albumresult, param['artist'], param['album'])
                    if albums:
                        param['url'] = albums[0]['url']
                    else:
                        return
                else:
                    return
            url = ALLMUSICDETAILS % param['url']
            albumscraper = allmusic_albumdetails
        result = get_data(url, json)
        if not result:
            return
        albumresults = albumscraper(result)
        if not albumresults:
            return
        details[site] = albumresults
        return details

    def compile_results(self, details):
        result = {}
        thumbs = []
        extras = []
        # merge metadata results, start with the least accurate sources
        if 'discogs' in details:
            for k, v in details['discogs'].items():
                if v:
                    result[k] = v
                if k == 'thumb' and v:
                    thumbs.append(v)
        if 'wikipedia' in details:
            for k, v in details['wikipedia'].items():
                if v:
                    result[k] = v
        if 'allmusic' in details:
            for k, v in details['allmusic'].items():
                if v:
                    result[k] = v
                if k == 'thumb' and v:
                    thumbs.append(v)
        if 'theaudiodb' in details:
            for k, v in details['theaudiodb'].items():
                if v:
                    result[k] = v
                if k == 'thumb' and v:
                    thumbs.append(v)
                if k == 'extras' and v:
                    extras.append(v)
        if 'musicbrainz' in details:
            for k, v in details['musicbrainz'].items():
                if v:
                    result[k] = v
        if 'coverarchive' in details:
            for k, v in details['coverarchive'].items():
                if v:
                    result[k] = v
                if k == 'thumb' and v:
                    thumbs.append(v)
                if k == 'extras' and v:
                    extras.append(v)
        # prefer artwork from fanarttv
        if 'fanarttv' in details:
            for k, v in details['fanarttv'].items():
                if v:
                    result[k] = v
                if k == 'thumb' and v:
                    thumbs.append(v)
                if k == 'extras' and v:
                    extras.append(v)
        # use musicbrainz artist as it provides the mbartistid (used for resolveid in the artist scraper)
        if 'musicbrainz' in details:
            result['artist'] = details['musicbrainz']['artist']
        # provide artwork from all scrapers for getthumb option
        if result:
            # thumb list from most accurate sources first
            thumbs.reverse()
            thumbnails = []
            for thumblist in thumbs:
                for item in thumblist:
                    thumbnails.append(item)
            # the order for extra art does not matter
            extraart = []
            for extralist in extras:
                for item in extralist:
                    extraart.append(item)
            # add the extra art to the end of the thumb list
            if extraart:
                thumbnails.extend(extraart)
            if thumbnails:
                result['thumb'] = thumbnails
        data = self.user_prefs(details, result)
        return data

    def user_prefs(self, details, result):
        # user preferences
        lang = 'description' + self.lang
        if self.review == 'theaudiodb' and 'theaudiodb' in details:
            if lang in details['theaudiodb']:
                result['description'] = details['theaudiodb'][lang]
            elif 'descriptionEN' in details['theaudiodb']:
                result['description'] = details['theaudiodb']['descriptionEN']
        elif (self.review in details) and ('description' in details[self.review]):
            result['description'] = details[self.review]['description']
        if (self.genre in details) and ('genre' in details[self.genre]):
            result['genre'] = details[self.genre]['genre']
        if (self.style in details) and ('styles' in details[self.style]):
            result['styles'] = details[self.style]['styles']
        if (self.mood in details) and ('moods' in details[self.mood]):
            result['moods'] = details[self.mood]['moods']
        if (self.theme in details) and ('themes' in details[self.theme]):
            result['themes'] = details[self.theme]['themes']
        if (self.rating in details) and ('rating' in details[self.rating]):
            result['rating'] = details[self.rating]['rating']
            result['votes'] = details[self.rating]['votes']
        return result

    def return_search(self, data):
        items = []
        for item in data:
            listitem = xbmcgui.ListItem(item['album'], offscreen=True)
            listitem.setArt({'thumb': item['thumb']})
            listitem.setProperty('album.artist', item['artist_description'])
            listitem.setProperty('album.year', item.get('year',''))
            listitem.setProperty('album.type', item.get('type',''))
            listitem.setProperty('album.releasestatus', item.get('releasestatus',''))
            listitem.setProperty('album.label', item.get('label',''))
            listitem.setProperty('relevance', item['relevance'])
            url = {'artist':item['artist_description'], 'album':item['album']}
            if 'mbalbumid' in item:
                url['mbalbumid'] = item['mbalbumid']
                url['mbreleasegroupid'] = item['mbreleasegroupid']
            if 'dcalbumid' in item:
                url['dcalbumid'] = item['dcalbumid']
            items.append((json.dumps(url), listitem, True))
        if items:
            xbmcplugin.addDirectoryItems(handle=int(sys.argv[1]), items=items)

    def return_nfourl(self, item):
        listitem = xbmcgui.ListItem(offscreen=True)
        xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]), url=json.dumps(item), listitem=listitem, isFolder=True)

    def return_resolved(self, item):
        listitem = xbmcgui.ListItem(path=json.dumps(item), offscreen=True)
        xbmcplugin.setResolvedUrl(handle=int(sys.argv[1]), succeeded=True, listitem=listitem)

    def return_details(self, item):
        if not 'album' in item:
            return
        listitem = xbmcgui.ListItem(item['album'], offscreen=True)
        if 'mbalbumid' in item:
            listitem.setProperty('album.musicbrainzid', item['mbalbumid'])
            listitem.setProperty('album.releaseid', item['mbalbumid'])
        if 'mbreleasegroupid' in item:
            listitem.setProperty('album.releasegroupid', item['mbreleasegroupid'])
        if 'scrapedmbid' in item:
            listitem.setProperty('album.scrapedmbid', item['scrapedmbid'])
        if 'artist' in item:
            listitem.setProperty('album.artists', str(len(item['artist'])))
            for count, artist in enumerate(item['artist']):
                listitem.setProperty('album.artist%i.name' % (count + 1), artist['artist'])
                listitem.setProperty('album.artist%i.musicbrainzid' % (count + 1), artist.get('mbartistid', ''))
                listitem.setProperty('album.artist%i.sortname' % (count + 1), artist.get('artistsort', ''))
        if 'genre' in item:
            listitem.setProperty('album.genre', item['genre'])
        if 'styles' in item:
            listitem.setProperty('album.styles', item['styles'])
        if 'moods' in item:
            listitem.setProperty('album.moods', item['moods'])
        if 'themes' in item:
            listitem.setProperty('album.themes', item['themes'])
        if 'description' in item:
            listitem.setProperty('album.review', item['description'])
        if 'releasedate' in item:
            listitem.setProperty('album.releasedate', item['releasedate'])
        if 'originaldate' in item:
            listitem.setProperty('album.originaldate', item['originaldate'])
        if 'releasestatus' in item:
            listitem.setProperty('album.releasestatus', item['releasestatus'])
        if 'artist_description' in item:
            listitem.setProperty('album.artist_description', item['artist_description'])
        if 'label' in item:
            listitem.setProperty('album.label', item['label'])
        if 'type' in item:
            listitem.setProperty('album.type', item['type'])
        if 'compilation' in item:
            listitem.setProperty('album.compilation', item['compilation'])
        if 'year' in item:
            listitem.setProperty('album.year', item['year'])
        if 'rating' in item:
            listitem.setProperty('album.rating', item['rating'])
        if 'votes' in item:
            listitem.setProperty('album.votes', item['votes'])
        if 'thumb' in item:
            listitem.setProperty('album.thumbs', str(len(item['thumb'])))
            for count, thumb in enumerate(item['thumb']):
                listitem.setProperty('album.thumb%i.url' % (count + 1), thumb['image'])
                listitem.setProperty('album.thumb%i.aspect' % (count + 1), thumb['aspect'])
                listitem.setProperty('album.thumb%i.preview' % (count + 1), thumb['preview'])
        xbmcplugin.setResolvedUrl(handle=int(sys.argv[1]), succeeded=True, listitem=listitem)



import re
try:
    from urlparse import parse_qsl
except ImportError: # py2 / py3
    from urllib.parse import parse_qsl

# get addon params from the plugin path querystring
def get_params(argv):
    result = {'handle': int(argv[0])}
    if len(argv) < 2 or not argv[1]:
        return result

    result.update(parse_qsl(argv[1].lstrip('?')))
    return result

def combine_scraped_details_info_and_ratings(original_details, additional_details):
    def update_or_set(details, key, value):
        if key in details:
            details[key].update(value)
        else:
            details[key] = value

    if additional_details:
        if additional_details.get('info'):
            update_or_set(original_details, 'info', additional_details['info'])
        if additional_details.get('ratings'):
            update_or_set(original_details, 'ratings', additional_details['ratings'])
    return original_details

def combine_scraped_details_available_artwork(original_details, additional_details):
    if additional_details and additional_details.get('available_art'):
        available_art = additional_details['available_art']
        if not original_details.get('available_art'):
            original_details['available_art'] = available_art
        else:
            for arttype, artlist in available_art.items():
                original_details['available_art'][arttype] = \
                    artlist + original_details['available_art'].get(arttype, [])

    return original_details

def find_uniqueids_in_text(input_text):
    result = {}
    res = re.search(r'(themoviedb.org/movie/)([0-9]+)', input_text)
    if (res):
        result['tmdb'] = res.group(2)
    res = re.search(r'imdb....?/title/tt([0-9]+)', input_text)
    if (res):
        result['imdb'] = 'tt' + res.group(1)
    else:
        res = re.search(r'imdb....?/Title\?t{0,2}([0-9]+)', input_text)
        if (res):
            result['imdb'] = 'tt' + res.group(1)
    return result



# -*- coding: utf-8 -*-

def musicbrainz_artistfind(data, artist):
    artists = []
    for item in data.get('artists',[]):
        artistdata = {}
        artistdata['artist'] = item['name']
        artistdata['thumb'] = ''
        artistdata['genre'] = ''
        artistdata['born'] = item['life-span'].get('begin', '')
        if 'type' in item:
            artistdata['type'] = item['type']
        if 'gender' in item:
            artistdata['gender'] = item['gender']
        if 'disambiguation' in item:
            artistdata['disambiguation'] = item['disambiguation']
        artistdata['mbartistid'] = item['id']
        if item.get('score',1):
            artistdata['relevance'] = str(item['score'] / 100.00)
        artists.append(artistdata)
    return artists

def musicbrainz_artistdetails(data):
    artistdata = {}
    artistdata['artist'] = data['name']
    artistdata['mbartistid'] = data['id']
    artistdata['type'] = data['type']
    artistdata['gender'] = data['gender']
    artistdata['disambiguation'] = data['disambiguation']
    if data.get('life-span','') and data.get('type',''):
        begin = data['life-span'].get('begin', '')
        end = data['life-span'].get('end', '')
        if data['type'] in ['Group', 'Orchestra', 'Choir']:
            artistdata['formed'] = begin
            artistdata['disbanded'] = end
        elif data['type'] in ['Person', 'Character']:
            artistdata['born'] = begin
            artistdata['died'] = end
    albums = []
    for item in data.get('release-groups',[]):
        albumdata = {}
        albumdata['title'] = item.get('title','')
        albumdata['year'] = item.get('first-release-date','')
        albumdata['musicbrainzreleasegroupid'] = item.get('id','')
        albums.append(albumdata)
    if albums:
        artistdata['albums'] = albums
    for item in data['relations']:
        if item['type'] == 'allmusic':
            artistdata['allmusic'] = item['url']['resource']
        elif item['type'] == 'discogs':
            dataid = item['url']['resource'].rsplit('/', 1)[1]
            artistdata['discogs'] = dataid
        elif item['type'] == 'wikidata':
            dataid = item['url']['resource'].rsplit('/', 1)[1]
            artistdata['wikidata'] = dataid
        elif item['type'] == 'wikipedia':
            dataid = item['url']['resource'].rsplit('/', 1)[1]
            artistdata['wikipedia'] = dataid
    return artistdata



# -*- coding: UTF-8 -*-
#
# Copyright (C) 2020, Team Kodi
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
# pylint: disable=missing-docstring

"""Functions to interact with Trakt API"""

from __future__ import absolute_import, unicode_literals

from . import api_utils, settings
from .utils import logger
try:
    from typing import Text, Optional, Union, List, Dict, Any  # pylint: disable=unused-import
    InfoType = Dict[Text, Any]  # pylint: disable=invalid-name
except ImportError:
    pass


HEADERS = (
    ('User-Agent', 'Kodi TV Show scraper by Team Kodi; contact pkscout@kodi.tv'),
    ('Accept', 'application/json'),
    ('trakt-api-key', settings.TRAKT_CLOWNCAR),
    ('trakt-api-version', '2'),
    ('Content-Type', 'application/json'),
)
api_utils.set_headers(dict(HEADERS))

SHOW_URL = 'https://api.trakt.tv/shows/{}'
EP_URL = SHOW_URL + '/seasons/{}/episodes/{}/ratings'


def get_details(imdb_id, season=None, episode=None):
    result = {}
    if season and episode:
        url = EP_URL.format(imdb_id, season, episode)
        params = None
    else:
        url = SHOW_URL.format(imdb_id)
        params = {'extended': 'full'}
    resp = api_utils.load_info(url, params=params, default={}, verboselog=settings.VERBOSELOG)
    rating =resp.get('rating')
    votes = resp.get('votes')
    if votes and rating:
        result['ratings'] = {'trakt': {'votes': votes, 'rating': rating}}
    return result



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2013 Team XBMC
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# PS3 Remote and Controller Keymaps

keymap_remote = {
    "16": 'power'    ,#EJECT
    "64": None       ,#AUDIO
    "65": None       ,#ANGLE
    "63": 'subtitle' ,#SUBTITLE
    "0f": None       ,#CLEAR
    "28": None       ,#TIME

    "00": 'one'   ,#1
    "01": 'two'   ,#2
    "02": 'three' ,#3
    "03": 'four'  ,#4
    "04": 'five'  ,#5
    "05": 'six'   ,#6
    "06": 'seven' ,#7
    "07": 'eight' ,#8
    "08": 'nine'  ,#9
    "09": 'zero'  ,#0

    "81": 'mytv'       ,#RED
    "82": 'mymusic'    ,#GREEN
    "80": 'mypictures' ,#BLUE
    "83": 'myvideo'    ,#YELLOW

    "70": 'display'  ,#DISPLAY
    "1a": None       ,#TOP MENU
    "40": 'menu'     ,#POP UP/MENU
    "0e": None       ,#RETURN

    "5c": 'menu'    ,#OPTIONS/TRIANGLE
    "5d": 'back'    ,#BACK/CIRCLE
    "5e": 'info'    ,#X
    "5f": 'title'   ,#VIEW/SQUARE

    "54": 'up'     ,#UP
    "55": 'right'  ,#RIGHT
    "56": 'down'   ,#DOWN
    "57": 'left'   ,#LEFT
    "0b": 'select' ,#ENTER

    "5a": 'volumeplus'  ,#L1
    "58": 'volumeminus' ,#L2
    "51": 'Mute'        ,#L3
    "5b": 'pageplus'    ,#R1
    "59": 'pageminus'   ,#R2
    "52": None          ,#R3

    "43": None          ,#PLAYSTATION
    "50": None          ,#SELECT
    "53": None          ,#START

    "33": 'reverse'   ,#<-SCAN
    "34": 'forward'   ,#  SCAN->
    "30": 'skipminus' ,#PREV
    "31": 'skipplus'  ,#NEXT
    "60": None        ,#<-SLOW/STEP
    "61": None        ,#  SLOW/STEP->
    "32": 'play'      ,#PLAY
    "38": 'stop'      ,#STOP
    "39": 'pause'     ,#PAUSE
    }




#! /usr/bin/env python

import os
import sys
import getopt
import subprocess


configs = ['Debug', 'Release']
solutions = ['../../../Build/Targets/x86-microsoft-win32-vs2008/Platinum.sln']

try:
    opts, args = getopt.getopt(sys.argv[1:], "b:rc")
except getopt.GetoptError, (msg, opt):
    print 'No build_config, defaulting to build all'
    
for opt, arg in opts:
    if opt == '-b':
        config = arg
        
        
def CallVsMake(sln, cfg):
    cmd = 'python VsMake.py -s %s -b %s' % (sln, cfg)
    print cmd
    retVal = subprocess.call(cmd.split())
    if retVal != 0:
        sys.exit(retVal)  
      
for sln in solutions:
    if 'config' not in locals() and 'config' not in globals():
        print '************ Building all configurations **************'
        for cfg in configs:
            CallVsMake(sln, cfg)
    else:
        print '************ Building configuration=' + config + ' ****************'
        CallVsMake(sln, config)
		    		


# -*- coding: utf-8 -*-

def fanarttv_albumart(data):
    if 'albums' in data:
        albumdata = {}
        thumbs = []
        extras = []
        discs = {}
        for mbid, art in data['albums'].items():
            if 'albumcover' in art:
                for thumb in art['albumcover']:
                    thumbdata = {}
                    thumbdata['image'] = thumb['url']
                    thumbdata['preview'] = thumb['url'].replace('/fanart/', '/preview/')
                    thumbdata['aspect'] = 'thumb'
                    thumbs.append(thumbdata)
            if 'cdart' in art:
                for cdart in art['cdart']:
                    extradata = {}
                    extradata['image'] = cdart['url']
                    extradata['preview'] = cdart['url'].replace('/fanart/', '/preview/')
                    extradata['aspect'] = 'discart'
                    extras.append(extradata)
                    # support for multi-disc albums
                    multidata = {}
                    num = cdart['disc']
                    multidata['image'] = cdart['url']
                    multidata['preview'] = cdart['url'].replace('/fanart/', '/preview/')
                    multidata['aspect'] = 'discart%s' % num
                    if not num in discs:
                        discs[num] = [multidata]
                    else:
                        discs[num].append(multidata)
        if thumbs:
            albumdata['thumb'] = thumbs
        # only return for multi-discs, not single discs
        if len(discs) > 1:
            for k, v in discs.items():
                for item in v:
                    extras.append(item)
        if extras:
            albumdata['extras'] = extras
        return albumdata



# -*- coding: UTF-8 -*-
#
# Copyright (C) 2020, Team Kodi
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
# pylint: disable=missing-docstring
#
# This is based on the metadata.tvmaze scrapper by Roman Miroshnychenko aka Roman V.M.

"""Cache-related functionality"""

from __future__ import absolute_import, unicode_literals

import os, pickle
import xbmc, xbmcvfs

from .utils import ADDON, logger

try:
    from typing import Optional, Text, Dict, Any  # pylint: disable=unused-import
except ImportError:
    pass



def _get_cache_directory():  # pylint: disable=missing-docstring
    # type: () -> Text
    temp_dir = xbmcvfs.translatePath('special://temp')
    cache_dir = os.path.join(temp_dir, 'scrapers', ADDON.getAddonInfo('id'))
    if not xbmcvfs.exists(cache_dir):
        xbmcvfs.mkdir(cache_dir)
    logger.debug('the cache dir is ' + cache_dir)
    return cache_dir


CACHE_DIR = _get_cache_directory()  # type: Text


def cache_show_info(show_info):
    # type: (Dict[Text, Any]) -> None
    """
    Save show_info dict to cache
    """
    file_name = str(show_info['id']) + '.pickle'
    cache = {
        'show_info': show_info
    }
    with open(os.path.join(CACHE_DIR, file_name), 'wb') as fo:
        pickle.dump(cache, fo, protocol=2)


def load_show_info_from_cache(show_id):
    # type: (Text) -> Optional[Dict[Text, Any]]
    """
    Load show info from a local cache

    :param show_id: show ID on TVmaze
    :return: show_info dict or None
    """
    file_name = str(show_id) + '.pickle'
    try:
        with open(os.path.join(CACHE_DIR, file_name), 'rb') as fo:
            load_kwargs = {}
            load_kwargs['encoding'] = 'bytes'
            cache = pickle.load(fo, **load_kwargs)
        return cache['show_info']
    except (IOError, pickle.PickleError) as exc:
        logger.debug('Cache message: {} {}'.format(type(exc), exc))
        return None



import json
import sys
import xbmc
import xbmcaddon
import xbmcgui
import xbmcplugin

from lib.tmdbscraper.tmdb import TMDBMovieScraper
from lib.tmdbscraper.fanarttv import get_details as get_fanarttv_artwork
from lib.tmdbscraper.imdbratings import get_details as get_imdb_details
from lib.tmdbscraper.traktratings import get_trakt_ratinginfo
from scraper_datahelper import combine_scraped_details_info_and_ratings, \
    combine_scraped_details_available_artwork, find_uniqueids_in_text, get_params
from scraper_config import configure_scraped_details, PathSpecificSettings, \
    configure_tmdb_artwork, is_fanarttv_configured

ADDON_SETTINGS = xbmcaddon.Addon()
ID = ADDON_SETTINGS.getAddonInfo('id')

def log(msg, level=xbmc.LOGDEBUG):
    xbmc.log(msg='[{addon}]: {msg}'.format(addon=ID, msg=msg), level=level)

def get_tmdb_scraper(settings):
    language = settings.getSettingString('language')
    certcountry = settings.getSettingString('tmdbcertcountry')
    return TMDBMovieScraper(ADDON_SETTINGS, language, certcountry)

def search_for_movie(title, year, handle, settings):
    log("Find movie with title '{title}' from year '{year}'".format(title=title, year=year), xbmc.LOGINFO)
    title = _strip_trailing_article(title)
    search_results = get_tmdb_scraper(settings).search(title, year)
    if not search_results:
        return
    if 'error' in search_results:
        header = "The Movie Database Python error searching with web service TMDB"
        xbmcgui.Dialog().notification(header, search_results['error'], xbmcgui.NOTIFICATION_WARNING)
        log(header + ': ' + search_results['error'], xbmc.LOGWARNING)
        return

    for movie in search_results:
        listitem = _searchresult_to_listitem(movie)
        uniqueids = {'tmdb': str(movie['id'])}
        xbmcplugin.addDirectoryItem(handle=handle, url=build_lookup_string(uniqueids),
            listitem=listitem, isFolder=True)

_articles = [prefix + article for prefix in (', ', ' ') for article in ("the", "a", "an")]
def _strip_trailing_article(title):
    title = title.lower()
    for article in _articles:
        if title.endswith(article):
            return title[:-len(article)]
    return title

def _searchresult_to_listitem(movie):
    movie_info = {'title': movie['title']}
    movie_label = movie['title']

    movie_year = movie['release_date'].split('-')[0] if movie.get('release_date') else None
    if movie_year:
        movie_label += ' ({})'.format(movie_year)
        movie_info['year'] = movie_year

    listitem = xbmcgui.ListItem(movie_label, offscreen=True)

    listitem.setInfo('video', movie_info)
    if movie['poster_path']:
        listitem.setArt({'thumb': movie['poster_path']})

    return listitem

# Low limit because a big list of artwork can cause trouble in some cases
# (a column can be too large for the MySQL integration),
# and how useful is a big list anyway? Not exactly rhetorical, this is an experiment.
IMAGE_LIMIT = 10

def add_artworks(listitem, artworks):
    for arttype, artlist in artworks.items():
        if arttype == 'fanart':
            continue
        for image in artlist[:IMAGE_LIMIT]:
            listitem.addAvailableArtwork(image['url'], arttype)

    fanart_to_set = [{'image': image['url'], 'preview': image['preview']}
        for image in artworks['fanart'][:IMAGE_LIMIT]]
    listitem.setAvailableFanart(fanart_to_set)

def get_details(input_uniqueids, handle, settings):
    if not input_uniqueids:
        return False
    details = get_tmdb_scraper(settings).get_details(input_uniqueids)
    if not details:
        return False
    if 'error' in details:
        header = "The Movie Database Python error with web service TMDB"
        xbmcgui.Dialog().notification(header, details['error'], xbmcgui.NOTIFICATION_WARNING)
        log(header + ': ' + details['error'], xbmc.LOGWARNING)
        return False

    details = configure_tmdb_artwork(details, settings)

    if settings.getSettingString('RatingS') == 'IMDb' or settings.getSettingBool('imdbanyway'):
        imdbinfo = get_imdb_details(details['uniqueids'])
        if 'error' in imdbinfo:
            header = "The Movie Database Python error with website IMDB"
            log(header + ': ' + imdbinfo['error'], xbmc.LOGWARNING)
        else:
            details = combine_scraped_details_info_and_ratings(details, imdbinfo)

    if settings.getSettingString('RatingS') == 'Trakt' or settings.getSettingBool('traktanyway'):
        traktinfo = get_trakt_ratinginfo(details['uniqueids'])
        details = combine_scraped_details_info_and_ratings(details, traktinfo)

    if is_fanarttv_configured(settings):
        fanarttv_info = get_fanarttv_artwork(details['uniqueids'],
            settings.getSettingString('fanarttv_clientkey'),
            settings.getSettingString('fanarttv_language'),
            details['_info']['set_tmdbid'])
        details = combine_scraped_details_available_artwork(details, fanarttv_info)

    details = configure_scraped_details(details, settings)

    listitem = xbmcgui.ListItem(details['info']['title'], offscreen=True)
    listitem.setInfo('video', details['info'])
    listitem.setCast(details['cast'])
    listitem.setUniqueIDs(details['uniqueids'], 'tmdb')
    add_artworks(listitem, details['available_art'])

    for rating_type, value in details['ratings'].items():
        if 'votes' in value:
            listitem.setRating(rating_type, value['rating'], value['votes'], value['default'])
        else:
            listitem.setRating(rating_type, value['rating'], defaultt=value['default'])

    xbmcplugin.setResolvedUrl(handle=handle, succeeded=True, listitem=listitem)
    return True

def find_uniqueids_in_nfo(nfo, handle):
    uniqueids = find_uniqueids_in_text(nfo)
    if uniqueids:
        listitem = xbmcgui.ListItem(offscreen=True)
        xbmcplugin.addDirectoryItem(
            handle=handle, url=build_lookup_string(uniqueids), listitem=listitem, isFolder=True)

def build_lookup_string(uniqueids):
    return json.dumps(uniqueids)

def parse_lookup_string(uniqueids):
    try:
        return json.loads(uniqueids)
    except ValueError:
        log("Can't parse this lookup string, is it from another add-on?\n" + uniqueids, xbmc.LOGWARNING)
        return None

def run():
    params = get_params(sys.argv[1:])
    enddir = True
    if 'action' in params:
        settings = ADDON_SETTINGS if not params.get('pathSettings') else \
            PathSpecificSettings(json.loads(params['pathSettings']), lambda msg: log(msg, xbmc.LOGWARNING))
        action = params["action"]
        if action == 'find' and 'title' in params:
            search_for_movie(params["title"], params.get("year"), params['handle'], settings)
        elif action == 'getdetails' and 'url' in params:
            enddir = not get_details(parse_lookup_string(params["url"]), params['handle'], settings)
        elif action == 'NfoUrl' and 'nfo' in params:
            find_uniqueids_in_nfo(params["nfo"], params['handle'])
        else:
            log("unhandled action: " + action, xbmc.LOGWARNING)
    else:
        log("No action in 'params' to act on", xbmc.LOGWARNING)
    if enddir:
        xbmcplugin.endOfDirectory(params['handle'])

if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-
import sys
from urllib.parse import parse_qsl
from lib.scraper import Scraper


class Main:
    def __init__(self):
        action, key, artist, album, url, nfo, settings = self._parse_argv()
        Scraper(action, key, artist, album, url, nfo, settings)

    def _parse_argv(self):
        params = dict(parse_qsl(sys.argv[2].lstrip('?')))
        # actions: find, resolveid, NfoUrl, getdetails
        action = params['action']
        # key: musicbrainz id
        key = params.get('key', '')
        # artist: artistname
        artist = params.get('artist', '')
        # album: albumtitle
        album = params.get('title', '')
        # url: provided by the scraper on previous run
        url = params.get('url', '')
        # nfo: musicbrainz url from .nfo file
        nfo = params.get('nfo', '')
        # path specific settings
        settings = params.get('pathSettings', {})
        return action, key, artist, album, url, nfo, settings


if (__name__ == '__main__'):
    Main()



# -*- coding: UTF-8 -*-
#
# Copyright (C) 2020, Team Kodi
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
# pylint: disable=missing-docstring
#
# This is based on the metadata.tvmaze scrapper by Roman Miroshnychenko aka Roman V.M.

"""Plugin route actions"""

from __future__ import absolute_import, unicode_literals

import sys, urllib.parse
import xbmcgui, xbmcplugin
from . import tmdb, data_utils
from .utils import logger, safe_get
try:
    from typing import Optional, Text, Union, ByteString  # pylint: disable=unused-import
except ImportError:
    pass

HANDLE = int(sys.argv[1])  # type: int


def find_show(title, year=None):
    # type: (Union[Text, bytes], Optional[Text]) -> None
    """Find a show by title"""
    if not isinstance(title, str):
        title = title.decode('utf-8')
    logger.debug('Searching for TV show {} ({})'.format(title, year))
    search_results = tmdb.search_show(title, year)
    for search_result in search_results:
        show_name = search_result['name']
        if safe_get(search_result, 'first_air_date') is not None:
            show_name += ' ({})'.format(search_result['first_air_date'][:4])
        list_item = xbmcgui.ListItem(show_name, offscreen=True)
        show_info = search_result
        list_item = data_utils.add_main_show_info(list_item, show_info, full_info=False)
        # Below "url" is some unique ID string (may be an actual URL to a show page)
        # that is used to get information about a specific TV show.
        xbmcplugin.addDirectoryItem(
            HANDLE,
            url=str(search_result['id']),
            listitem=list_item,
            isFolder=True
        )


def get_show_id_from_nfo(nfo):
    # type: (Text) -> None
    """
    Get show ID by NFO file contents

    This function is called first instead of find_show
    if a NFO file is found in a TV show folder.

    :param nfo: the contents of a NFO file
    """
    if isinstance(nfo, bytes):
        nfo = nfo.decode('utf-8', 'replace')
    logger.debug('Parsing NFO file:\n{}'.format(nfo))
    parse_result, named_seasons = data_utils.parse_nfo_url(nfo)
    if parse_result:
        if parse_result.provider == 'themoviedb':
            show_info = tmdb.load_show_info(parse_result.show_id, ep_grouping=parse_result.ep_grouping, named_seasons=named_seasons)
        else:
            show_info = None
        if show_info is not None:
            list_item = xbmcgui.ListItem(show_info['name'], offscreen=True)
            # "url" is some string that unique identifies a show.
            # It may be an actual URL of a TV show page.
            xbmcplugin.addDirectoryItem(
                HANDLE,
                url=str(show_info['id']),
                listitem=list_item,
                isFolder=True
            )


def get_details(show_id):
    # type: (Text) -> None
    """Get details about a specific show"""
    logger.debug('Getting details for show id {}'.format(show_id))
    show_info = tmdb.load_show_info(show_id)
    if show_info is not None:
        list_item = xbmcgui.ListItem(show_info['name'], offscreen=True)
        list_item = data_utils.add_main_show_info(list_item, show_info, full_info=True)
        xbmcplugin.setResolvedUrl(HANDLE, True, list_item)
    else:
        xbmcplugin.setResolvedUrl(HANDLE, False, xbmcgui.ListItem(offscreen=True))


def get_episode_list(show_id):  # pylint: disable=missing-docstring
    # type: (Text) -> None
    logger.debug('Getting episode list for show id {}'.format(show_id))
    if not show_id.isdigit():
        # Kodi has a bug: when a show directory contains an XML NFO file with
        # episodeguide URL, that URL is always passed here regardless of
        # the actual parsing result in get_show_from_nfo()
        parse_result, named_seasons = data_utils.parse_nfo_url(show_id)
        if not parse_result:
            return
        if parse_result.provider == 'themoviedb' or parse_result.provider == 'tmdb':
            show_info = tmdb.load_show_info(parse_result.show_id)
        else:
            return
    else:
        show_info = tmdb.load_show_info(show_id)
    if show_info is not None:
        theindex = 0
        for episode in show_info['episodes']:
            epname = episode.get('name', 'Episode ' + str(episode['episode_number']))
            list_item = xbmcgui.ListItem(epname, offscreen=True)
            list_item = data_utils.add_episode_info(list_item, episode, full_info=False)
            encoded_ids = urllib.parse.urlencode(
                {'show_id': str(show_info['id']), 'episode_id': str(theindex)}
            )
            theindex = theindex + 1
            # Below "url" is some unique ID string (may be an actual URL to an episode page)
            # that allows to retrieve information about a specific episode.
            url = urllib.parse.quote(encoded_ids)
            xbmcplugin.addDirectoryItem(
                HANDLE,
                url=url,
                listitem=list_item,
                isFolder=True
            )


def get_episode_details(encoded_ids):  # pylint: disable=missing-docstring
    # type: (Text) -> None
    encoded_ids = urllib.parse.unquote(encoded_ids)
    decoded_ids = dict(urllib.parse.parse_qsl(encoded_ids))
    logger.debug('Getting episode details for {}'.format(decoded_ids))
    episode_info = tmdb.load_episode_info(
        decoded_ids['show_id'], decoded_ids['episode_id']
    )
    if episode_info:
        list_item = xbmcgui.ListItem(episode_info['name'], offscreen=True)
        list_item = data_utils.add_episode_info(list_item, episode_info, full_info=True)
        xbmcplugin.setResolvedUrl(HANDLE, True, list_item)
    else:
        xbmcplugin.setResolvedUrl(HANDLE, False, xbmcgui.ListItem(offscreen=True))


def get_artwork(show_id):
    # type: (Text) -> None
    """
    Get available artwork for a show

    :param show_id: default unique ID set by setUniqueIDs() method
    """
    if not show_id:
      return
    logger.debug('Getting artwork for show ID {}'.format(show_id))
    show_info = tmdb.load_show_info(show_id)
    if show_info is not None:
        list_item = xbmcgui.ListItem(show_info['name'], offscreen=True)
        list_item = data_utils.set_show_artwork(show_info, list_item)
        xbmcplugin.setResolvedUrl(HANDLE, True, list_item)
    else:
        xbmcplugin.setResolvedUrl(HANDLE, False, xbmcgui.ListItem(offscreen=True))


def router(paramstring):
    # type: (Text) -> None
    """
    Route addon calls

    :param paramstring: url-encoded query string
    :raises RuntimeError: on unknown call action
    """
    params = dict(urllib.parse.parse_qsl(paramstring))
    logger.debug('Called addon with params: {}'.format(sys.argv))
    if params['action'] == 'find':
        logger.debug('performing find action')
        find_show(params['title'], params.get('year'))
    elif params['action'].lower() == 'nfourl':
        logger.debug('performing nfourl action')
        get_show_id_from_nfo(params['nfo'])
    elif params['action'] == 'getdetails':
        logger.debug('performing getdetails action')
        get_details(params['url'])
    elif params['action'] == 'getepisodelist':
        logger.debug('performing getepisodelist action')
        get_episode_list(params['url'])
    elif params['action'] == 'getepisodedetails':
        logger.debug('performing getepisodedetails action')
        get_episode_details(params['url'])
    elif params['action'] == 'getartwork':
        logger.debug('performing getartwork action')
        get_artwork(params.get('id'))
    else:
        raise RuntimeError('Invalid addon call: {}'.format(sys.argv))
    xbmcplugin.endOfDirectory(HANDLE)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2008-2013 Team XBMC
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Implementation of XBMC's UDP based input system.

A set of classes that abstract the various packets that the event server
currently supports. In addition, there's also a class, XBMCClient, that
provides functions that sends the various packets. Use XBMCClient if you
don't need complete control over packet structure.

The basic workflow involves:

1. Send a HELO packet
2. Send x number of valid packets
3. Send a BYE packet

IMPORTANT NOTE ABOUT TIMEOUTS:
A client is considered to be timed out if XBMC doesn't received a packet
at least once every 60 seconds. To "ping" XBMC with an empty packet use
PacketPING or XBMCClient.ping(). See the documentation for details.
"""

from __future__ import unicode_literals, print_function, absolute_import, division

__author__  = "d4rk@xbmc.org"
__version__ = "0.1.0"

import sys
if sys.version_info.major == 2:
    str = unicode
from struct import pack
from socket import socket, AF_INET, SOCK_DGRAM, SOL_SOCKET, SO_BROADCAST
import time

MAX_PACKET_SIZE  = 1024
HEADER_SIZE      = 32
MAX_PAYLOAD_SIZE = MAX_PACKET_SIZE - HEADER_SIZE
UNIQUE_IDENTIFICATION = (int)(time.time())

PT_HELO          = 0x01
PT_BYE           = 0x02
PT_BUTTON        = 0x03
PT_MOUSE         = 0x04
PT_PING          = 0x05
PT_BROADCAST     = 0x06
PT_NOTIFICATION  = 0x07
PT_BLOB          = 0x08
PT_LOG           = 0x09
PT_ACTION        = 0x0A
PT_DEBUG         = 0xFF

ICON_NONE = 0x00
ICON_JPEG = 0x01
ICON_PNG  = 0x02
ICON_GIF  = 0x03

BT_USE_NAME   = 0x01
BT_DOWN       = 0x02
BT_UP         = 0x04
BT_USE_AMOUNT = 0x08
BT_QUEUE      = 0x10
BT_NO_REPEAT  = 0x20
BT_VKEY       = 0x40
BT_AXIS       = 0x80
BT_AXISSINGLE = 0x100

MS_ABSOLUTE = 0x01

LOGDEBUG   = 0x00
LOGINFO    = 0x01
LOGNOTICE  = 0x02
LOGWARNING = 0x03
LOGERROR   = 0x04
LOGSEVERE  = 0x05
LOGFATAL   = 0x06
LOGNONE    = 0x07

ACTION_EXECBUILTIN = 0x01
ACTION_BUTTON      = 0x02

######################################################################
# Helper Functions
######################################################################

def format_string(msg):
    """ """
    return msg.encode('utf-8') + b"\0"

def format_uint32(num):
    """ """
    return pack ("!I", num)

def format_uint16(num):
    """ """
    if num<0:
        num = 0
    elif num>65535:
        num = 65535
    return pack ("!H", num)


######################################################################
#  Packet Classes
######################################################################

class Packet:
    """Base class that implements a single event packet.

     - Generic packet structure (maximum 1024 bytes per packet)
     - Header is 32 bytes long, so 992 bytes available for payload
     - large payloads can be split into multiple packets using H4 and H5
       H5 should contain total no. of packets in such a case
     - H6 contains length of P1, which is limited to 992 bytes
     - if H5 is 0 or 1, then H4 will be ignored (single packet msg)
     - H7 must be set to zeros for now

         -----------------------------
         | -H1 Signature ("XBMC")    | - 4  x CHAR                4B
         | -H2 Version (eg. 2.0)     | - 2  x UNSIGNED CHAR       2B
         | -H3 PacketType            | - 1  x UNSIGNED SHORT      2B
         | -H4 Sequence number       | - 1  x UNSIGNED LONG       4B
         | -H5 No. of packets in msg | - 1  x UNSIGNED LONG       4B
         | -H7 Client's unique token | - 1  x UNSIGNED LONG       4B
         | -H8 Reserved              | - 10 x UNSIGNED CHAR      10B
         |---------------------------|
         | -P1 payload               | -
         -----------------------------
    """
    def __init__(self):
        self.sig = b"XBMC"
        self.minver = 0
        self.majver = 2
        self.seq = 1
        self.maxseq = 1
        self.payloadsize = 0
        self.uid = UNIQUE_IDENTIFICATION
        self.reserved = b"\0" * 10
        self.payload = b""

    def append_payload(self, blob):
        """Append to existing payload

        Arguments:
        blob -- binary data to append to the current payload
        """
        if isinstance(blob, str):
            blob = blob.encode()
        self.set_payload(self.payload + blob)


    def set_payload(self, payload):
        """Set the payload for this packet

        Arguments:
        payload -- binary data that contains the payload
        """
        if isinstance(payload, str):
            payload = payload.encode()
        self.payload = payload
        self.payloadsize = len(self.payload)
        self.maxseq = int((self.payloadsize + (MAX_PAYLOAD_SIZE - 1)) / MAX_PAYLOAD_SIZE)


    def num_packets(self):
        """ Return the number of packets required for payload """
        return self.maxseq

    def get_header(self, packettype=-1, seq=1, maxseq=1, payload_size=0):
        """Construct a header and return as string

        Keyword arguments:
        packettype -- valid packet types are PT_HELO, PT_BYE, PT_BUTTON,
                      PT_MOUSE, PT_PING, PT_BORADCAST, PT_NOTIFICATION,
                      PT_BLOB, PT_DEBUG
        seq -- the sequence of this packet for a multi packet message
               (default 1)
        maxseq -- the total number of packets for a multi packet message
                  (default 1)
        payload_size -- the size of the payload of this packet (default 0)
        """
        if packettype < 0:
            packettype = self.packettype
        header = self.sig
        header += chr(self.majver).encode()
        header += chr(self.minver).encode()
        header += format_uint16(packettype)
        header += format_uint32(seq)
        header += format_uint32(maxseq)
        header += format_uint16(payload_size)
        header += format_uint32(self.uid)
        header += self.reserved
        return header

    def get_payload_size(self, seq):
        """Returns the calculated payload size for the particular packet

        Arguments:
        seq -- the sequence number
        """
        if self.maxseq == 1:
            return self.payloadsize

        if seq < self.maxseq:
            return MAX_PAYLOAD_SIZE

        return self.payloadsize % MAX_PAYLOAD_SIZE


    def get_udp_message(self, packetnum=1):
        """Construct the UDP message for the specified packetnum and return
        as string

        Keyword arguments:
        packetnum -- the packet no. for which to construct the message
                     (default 1)
        """
        if packetnum > self.num_packets() or packetnum < 1:
            return b""
        header = b""
        if packetnum==1:
            header = self.get_header(self.packettype, packetnum, self.maxseq,
                                     self.get_payload_size(packetnum))
        else:
            header = self.get_header(PT_BLOB, packetnum, self.maxseq,
                                     self.get_payload_size(packetnum))

        payload = self.payload[ (packetnum-1) * MAX_PAYLOAD_SIZE :
                                (packetnum-1) * MAX_PAYLOAD_SIZE+
                                self.get_payload_size(packetnum) ]
        return header + payload

    def send(self, sock, addr, uid=UNIQUE_IDENTIFICATION):
        """Send the entire message to the specified socket and address.

        Arguments:
        sock -- datagram socket object (socket.socket)
        addr -- address, port pair (eg: ("127.0.0.1", 9777) )
        uid  -- unique identification
        """
        self.uid = uid
        for a in range ( 0, self.num_packets() ):
            sock.sendto(self.get_udp_message(a+1), addr)


class PacketHELO (Packet):
    """A HELO packet

    A HELO packet establishes a valid connection to XBMC. It is the
    first packet that should be sent.
    """
    def __init__(self, devicename=None, icon_type=ICON_NONE, icon_file=None):
        """
        Keyword arguments:
        devicename -- the string that identifies the client
        icon_type -- one of ICON_NONE, ICON_JPEG, ICON_PNG, ICON_GIF
        icon_file -- location of icon file with respect to current working
                     directory if icon_type is not ICON_NONE
        """
        Packet.__init__(self)
        self.packettype = PT_HELO
        self.icontype = icon_type
        self.set_payload ( format_string(devicename)[0:128] )
        self.append_payload( chr (icon_type) )
        self.append_payload( format_uint16 (0) ) # port no
        self.append_payload( format_uint32 (0) ) # reserved1
        self.append_payload( format_uint32 (0) ) # reserved2
        if icon_type != ICON_NONE and icon_file:
            with open(icon_file, 'rb') as f:
                self.append_payload(f.read())


class PacketNOTIFICATION (Packet):
    """A NOTIFICATION packet

    This packet displays a notification window in XBMC. It can contain
    a caption, a message and an icon.
    """
    def __init__(self, title, message, icon_type=ICON_NONE, icon_file=None):
        """
        Keyword arguments:
        title -- the notification caption / title
        message -- the main text of the notification
        icon_type -- one of ICON_NONE, ICON_JPEG, ICON_PNG, ICON_GIF
        icon_file -- location of icon file with respect to current working
                     directory if icon_type is not ICON_NONE
        """
        Packet.__init__(self)
        self.packettype = PT_NOTIFICATION
        self.title = title
        self.message = message
        self.set_payload ( format_string(title) )
        self.append_payload( format_string(message) )
        self.append_payload( chr (icon_type) )
        self.append_payload( format_uint32 (0) ) # reserved
        if icon_type != ICON_NONE and icon_file:
            with open(icon_file, 'rb') as f:
                self.append_payload(f.read())

class PacketBUTTON (Packet):
    """A BUTTON packet

    A button packet send a key press or release event to XBMC
    """
    def __init__(self, code=0, repeat=1, down=1, queue=0,
                 map_name="", button_name="", amount=0, axis=0):
        """
        Keyword arguments:
        code -- raw button code (default: 0)
        repeat -- this key press should repeat until released (default: 1)
                  Note that queued pressed cannot repeat.
        down -- if this is 1, it implies a press event, 0 implies a release
                event. (default: 1)
        queue -- a queued key press means that the button event is
                 executed just once after which the next key press is
                 processed. It can be used for macros. Currently there
                 is no support for time delays between queued presses.
                 (default: 0)
        map_name -- a combination of map_name and button_name refers to a
                    mapping in the user's Keymap.xml or Lircmap.xml.
                    map_name can be one of the following:
                    "KB" => standard keyboard map ( <keyboard> section )
                    "XG" => xbox gamepad map ( <gamepad> section )
                    "R1" => xbox remote map ( <remote> section )
                    "R2" => xbox universal remote map ( <universalremote>
                            section )
                    "LI:devicename" => LIRC remote map where 'devicename' is the
                    actual device's name
        button_name -- a button name defined in the map specified in map_name.
                       For example, if map_name is "KB" referring to the
                       <keyboard> section in Keymap.xml then, valid
                       button_names include "printscreen", "minus", "x", etc.
        amount -- unimplemented for now; in the future it will be used for
                  specifying magnitude of analog key press events
        """
        Packet.__init__(self)
        self.flags = 0
        self.packettype = PT_BUTTON
        if type (code ) == str:
            code = ord(code)

        # assign code only if we don't have a map and button name
        if not (map_name and button_name):
            self.code = code
        else:
            self.flags |= BT_USE_NAME
            self.code = 0
        if (amount != None):
            self.flags |= BT_USE_AMOUNT
            self.amount = int(amount)
        else:
            self.amount = 0

        if down:
            self.flags |= BT_DOWN
        else:
            self.flags |= BT_UP
        if not repeat:
            self.flags |= BT_NO_REPEAT
        if queue:
            self.flags |= BT_QUEUE
        if axis == 1:
            self.flags |= BT_AXISSINGLE
        elif axis == 2:
            self.flags |= BT_AXIS

        self.set_payload ( format_uint16(self.code) )
        self.append_payload( format_uint16(self.flags) )
        self.append_payload( format_uint16(self.amount) )
        self.append_payload( format_string (map_name) )
        self.append_payload( format_string (button_name) )

class PacketMOUSE (Packet):
    """A MOUSE packet

    A MOUSE packets sets the mouse position in XBMC
    """
    def __init__(self, x, y):
        """
        Arguments:
        x -- horizontal position ranging from 0 to 65535
        y -- vertical position ranging from 0 to 65535

        The range will be mapped to the screen width and height in XBMC
        """
        Packet.__init__(self)
        self.packettype = PT_MOUSE
        self.flags = MS_ABSOLUTE
        self.append_payload( chr (self.flags) )
        self.append_payload( format_uint16(x) )
        self.append_payload( format_uint16(y) )

class PacketBYE (Packet):
    """A BYE packet

    A BYE packet terminates the connection to XBMC.
    """
    def __init__(self):
        Packet.__init__(self)
        self.packettype = PT_BYE


class PacketPING (Packet):
    """A PING packet

    A PING packet tells XBMC that the client is still alive. All valid
    packets act as ping (not just this one). A client needs to ping
    XBMC at least once in 60 seconds or it will time out.
    """
    def __init__(self):
        Packet.__init__(self)
        self.packettype = PT_PING

class PacketLOG (Packet):
    """A LOG packet

    A LOG packet tells XBMC to log the message to xbmc.log with the loglevel as specified.
    """
    def __init__(self, loglevel=0, logmessage="", autoprint=True):
        """
        Keyword arguments:
        loglevel -- the loglevel, follows XBMC standard.
        logmessage -- the message to log
        autoprint -- if the logmessage should automatically be printed to stdout
        """
        Packet.__init__(self)
        self.packettype = PT_LOG
        self.append_payload( chr (loglevel) )
        self.append_payload( format_string(logmessage) )
        if (autoprint):
            print(logmessage)

class PacketACTION (Packet):
    """An ACTION packet

    An ACTION packet tells XBMC to do the action specified, based on the type it knows were it needs to be sent.
    The idea is that this will be as in scripting/skining and keymapping, just triggered from afar.
    """
    def __init__(self, actionmessage="", actiontype=ACTION_EXECBUILTIN):
        """
        Keyword arguments:
        loglevel -- the loglevel, follows XBMC standard.
        logmessage -- the message to log
        autoprint -- if the logmessage should automatically be printed to stdout
        """
        Packet.__init__(self)
        self.packettype = PT_ACTION
        self.append_payload( chr (actiontype) )
        self.append_payload( format_string(actionmessage) )

######################################################################
# XBMC Client Class
######################################################################

class XBMCClient:
    """An XBMC event client"""

    def __init__(self, name ="", icon_file=None, broadcast=False, uid=UNIQUE_IDENTIFICATION,
                 ip="127.0.0.1"):
        """
        Keyword arguments:
        name -- Name of the client
        icon_file -- location of an icon file, if any (png, jpg or gif)
        uid  -- unique identification
        """
        self.name = str(name)
        self.icon_file = icon_file
        self.icon_type = self._get_icon_type(icon_file)
        self.ip = ip
        self.port = 9777
        self.sock = socket(AF_INET,SOCK_DGRAM)
        if broadcast:
            self.sock.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
        self.uid = uid


    def connect(self, ip=None, port=None):
        """Initialize connection to XBMC
        ip -- IP Address of XBMC
        port -- port that the event server on XBMC is listening on
        """
        if ip:
            self.ip = ip
        if port:
            self.port = int(port)
        self.addr = (self.ip, self.port)
        packet = PacketHELO(self.name, self.icon_type, self.icon_file)
        packet.send(self.sock, self.addr, self.uid)


    def close(self):
        """Close the current connection"""
        packet = PacketBYE()
        packet.send(self.sock, self.addr, self.uid)


    def ping(self):
        """Send a PING packet"""
        packet = PacketPING()
        packet.send(self.sock, self.addr, self.uid)


    def send_notification(self, title="", message="", icon_file=None):
        """Send a notification to the connected XBMC
        Keyword Arguments:
        title -- The title/heading for the notification
        message -- The message to be displayed
        icon_file -- location of an icon file, if any (png, jpg, gif)
        """
        self.connect()
        packet = PacketNOTIFICATION(title, message,
                                    self._get_icon_type(icon_file),
                                    icon_file)
        packet.send(self.sock, self.addr, self.uid)


    def send_keyboard_button(self, button=None):
        """Send a keyboard event to XBMC
        Keyword Arguments:
        button -- name of the keyboard button to send (same as in Keymap.xml)
        """
        if not button:
            return
        self.send_button(map="KB", button=button)


    def send_remote_button(self, button=None):
        """Send a remote control event to XBMC
        Keyword Arguments:
        button -- name of the remote control button to send (same as in Keymap.xml)
        """
        if not button:
            return
        self.send_button(map="R1", button=button)


    def release_button(self):
        """Release all buttons"""
        packet = PacketBUTTON(code=0x01, down=0)
        packet.send(self.sock, self.addr, self.uid)


    def send_button(self, map="", button="", amount=0):
        """Send a button event to XBMC
        Keyword arguments:
        map -- a combination of map_name and button_name refers to a
               mapping in the user's Keymap.xml or Lircmap.xml.
               map_name can be one of the following:
                   "KB" => standard keyboard map ( <keyboard> section )
                   "XG" => xbox gamepad map ( <gamepad> section )
                   "R1" => xbox remote map ( <remote> section )
                   "R2" => xbox universal remote map ( <universalremote>
                           section )
                   "LI:devicename" => LIRC remote map where 'devicename' is the
                                      actual device's name
        button -- a button name defined in the map specified in map, above.
                  For example, if map is "KB" referring to the <keyboard>
                  section in Keymap.xml then, valid buttons include
                  "printscreen", "minus", "x", etc.
        """
        packet = PacketBUTTON(map_name=str(map), button_name=str(button), amount=amount)
        packet.send(self.sock, self.addr, self.uid)

    def send_button_state(self, map="", button="", amount=0, down=0, axis=0):
        """Send a button event to XBMC
        Keyword arguments:
        map -- a combination of map_name and button_name refers to a
               mapping in the user's Keymap.xml or Lircmap.xml.
               map_name can be one of the following:
                   "KB" => standard keyboard map ( <keyboard> section )
                   "XG" => xbox gamepad map ( <gamepad> section )
                   "R1" => xbox remote map ( <remote> section )
                   "R2" => xbox universal remote map ( <universalremote>
                           section )
                   "LI:devicename" => LIRC remote map where 'devicename' is the
                                      actual device's name
        button -- a button name defined in the map specified in map, above.
                  For example, if map is "KB" referring to the <keyboard>
                  section in Keymap.xml then, valid buttons include
                  "printscreen", "minus", "x", etc.
        """
        if axis:
            down = int(amount != 0)

        packet = PacketBUTTON(map_name=str(map), button_name=str(button), amount=amount, down=down, queue=1, axis=axis)
        packet.send(self.sock, self.addr, self.uid)

    def send_mouse_position(self, x=0, y=0):
        """Send a mouse event to XBMC
        Keywords Arguments:
        x -- absolute x position of mouse ranging from 0 to 65535
             which maps to the entire screen width
        y -- same a 'x' but relates to the screen height
        """
        packet = PacketMOUSE(int(x), int(y))
        packet.send(self.sock, self.addr, self.uid)

    def send_log(self, loglevel=0, logmessage="", autoprint=True):
        """
        Keyword arguments:
        loglevel -- the loglevel, follows XBMC standard.
        logmessage -- the message to log
        autoprint -- if the logmessage should automatically be printed to stdout
        """
        packet = PacketLOG(loglevel, logmessage)
        packet.send(self.sock, self.addr, self.uid)

    def send_action(self, actionmessage="", actiontype=ACTION_EXECBUILTIN):
        """
        Keyword arguments:
        actionmessage -- the ActionString
        actiontype -- The ActionType the ActionString should be sent to.
        """
        packet = PacketACTION(actionmessage, actiontype)
        packet.send(self.sock, self.addr, self.uid)

    def _get_icon_type(self, icon_file):
        if icon_file:
            if icon_file.lower()[-3:] == "png":
                return ICON_PNG
            elif icon_file.lower()[-3:] == "gif":
                return ICON_GIF
            elif icon_file.lower()[-3:] == "jpg":
                return ICON_JPEG
        return ICON_NONE



# -*- coding: utf-8 -*-

"""

    Copyright (C) 2019 Team Kodi

    This file is part of service.xbmc.versioncheck

    SPDX-License-Identifier: GPL-3.0-or-later
    See LICENSES/GPL-3.0-or-later.txt for more information.

"""

__all__ = ['runner', 'version_check']



# -*- coding: utf-8 -*-

def musicbrainz_albumfind(data, artist, album):
    albums = []
    # count how often each releasegroup occurs in the release results
    # keep track of the release with the highest score and earliest releasedate in each releasegroup
    releasegroups = {}
    for item in data.get('releases'):
        mbid = item['id']
        score = item.get('score', 0)
        releasegroup = item['release-group']['id']
        if 'date' in item and item['date']:
            date = item['date'].replace('-','')
            if len(date) == 4:
                date = date + '9999'
        else:
            date = '99999999'
        if releasegroup in releasegroups:
            count = releasegroups[releasegroup][0] + 1
            topmbid = releasegroups[releasegroup][1]
            topdate = releasegroups[releasegroup][2]
            topscore = releasegroups[releasegroup][3]
            if date < topdate and score >= topscore:
                topdate = date
                topmbid = mbid
            releasegroups[releasegroup] = [count, topmbid, topdate, topscore]
        else:
            releasegroups[releasegroup] = [1, mbid, date, score]
    if releasegroups:
        # get the highest releasegroup count
        maxcount = max(releasegroups.values())[0]
        # get the releasegroup(s) that match this highest value
        topgroups = [k for k, v in releasegroups.items() if v[0] == maxcount]
    for item in data.get('releases'):
        # only use the 'top' release from each releasegroup
        if item['id'] != releasegroups[item['release-group']['id']][1]:
            continue
        albumdata = {}
        if item.get('artist-credit'):
            artists = []
            artistdisp = ""
            for artist in item['artist-credit']:
                artistdata = {}
                artistdata['artist'] = artist['artist']['name']
                artistdata['mbartistid'] = artist['artist']['id']
                artistdata['artistsort'] = artist['artist']['sort-name']
                artistdisp = artistdisp + artist['artist']['name']
                artistdisp = artistdisp + artist.get('joinphrase', '')
                artists.append(artistdata)
            albumdata['artist'] = artists
            albumdata['artist_description'] = artistdisp
        if item.get('label-info','') and item['label-info'][0].get('label','') and item['label-info'][0]['label'].get('name',''):
            albumdata['label'] = item['label-info'][0]['label']['name']
        albumdata['album'] = item['title']
        if item.get('date',''):
            albumdata['year'] = item['date'][:4]
        albumdata['thumb'] = 'https://coverartarchive.org/release-group/%s/front-250' % item['release-group']['id']
        if item.get('label-info','') and item['label-info'][0].get('label','') and item['label-info'][0]['label'].get('name',''):
            albumdata['label'] = item['label-info'][0]['label']['name']
        if item.get('status',''):
            albumdata['releasestatus'] = item['status']
        albumdata['type'] = item['release-group'].get('primary-type')
        albumdata['mbalbumid'] = item['id']
        albumdata['mbreleasegroupid'] = item['release-group']['id']
        if item.get('score'):
            releasescore = item['score'] / 100.0
            # if the release is in the releasegroup with most releases, it is considered the most accurate one
            # (this also helps with prefering official releases over bootlegs, assuming there are more variations of an official release than of a bootleg)
            if item['release-group']['id'] not in topgroups:
                releasescore -= 0.001
            # if the release is an album, prefer it over singles/ep's
            # (this needs to be the double of the above, as me might have just given the album a lesser score if the single happened to be in the topgroup)
            if item['release-group'].get('primary-type') != 'Album':
                releasescore -= 0.002
            albumdata['relevance'] = str(releasescore)
        albums.append(albumdata)
    return albums

def musicbrainz_albumlinks(data):
    albumlinks = {}
    if 'relations' in data and data['relations']:
        for item in data['relations']:
            if item['type'] == 'allmusic':
                albumlinks['allmusic'] = item['url']['resource']
            elif item['type'] == 'discogs':
                albumlinks['discogs'] = item['url']['resource'].rsplit('/', 1)[1]
            elif item['type'] == 'wikipedia':
                albumlinks['wikipedia'] = item['url']['resource'].rsplit('/', 1)[1]
            elif item['type'] == 'wikidata':
                albumlinks['wikidata'] = item['url']['resource'].rsplit('/', 1)[1]
    return albumlinks

def musicbrainz_albumdetails(data):
    albumdata = {}
    albumdata['album'] = data['title']
    albumdata['mbalbumid'] = data['id']
    if data.get('release-group',''):
        albumdata['mbreleasegroupid'] = data['release-group']['id']
        if data['release-group']['rating'] and data['release-group']['rating']['value']:
            albumdata['rating'] = str(int((float(data['release-group']['rating']['value']) * 2) + 0.5))
            albumdata['votes'] = str(data['release-group']['rating']['votes-count'])
        if data['release-group'].get('primary-type'):
            albumtypes = [data['release-group']['primary-type']] + data['release-group']['secondary-types']
            albumdata['type'] = ' / '.join(albumtypes)
            if 'Compilation' in albumtypes:
                albumdata['compilation'] = 'true'
        if data['release-group'].get('first-release-date',''):
            albumdata['originaldate'] = data['release-group']['first-release-date']
    if data.get('release-events',''):
        albumdata['year'] = data['release-events'][0]['date'][:4]
        albumdata['releasedate'] = data['release-events'][0]['date']
    if data.get('label-info','') and data['label-info'][0].get('label','') and data['label-info'][0]['label'].get('name',''):
        albumdata['label'] = data['label-info'][0]['label']['name']
    if data.get('status',''):
        albumdata['releasestatus'] = data['status']
    if data.get('artist-credit'):
        artists = []
        artistdisp = ''
        for artist in data['artist-credit']:
            artistdata = {}
            artistdata['artist'] = artist['name']
            artistdata['mbartistid'] = artist['artist']['id']
            artistdata['artistsort'] = artist['artist']['sort-name']
            artistdisp = artistdisp + artist['name']
            artistdisp = artistdisp + artist.get('joinphrase', '')
            artists.append(artistdata)
        albumdata['artist'] = artists
        albumdata['artist_description'] = artistdisp
    return albumdata

def musicbrainz_albumart(data):
    albumdata = {}
    thumbs = []
    extras = []
    for item in data['images']:
        if 'Front' in item['types']:
            thumbdata = {}
            thumbdata['image'] = item['image']
            thumbdata['preview'] = item['thumbnails']['small']
            thumbdata['aspect'] = 'thumb'
            thumbs.append(thumbdata)
        if 'Back' in item['types']:
            backdata = {}
            backdata['image'] = item['image']
            backdata['preview'] = item['thumbnails']['small']
            backdata['aspect'] = 'back'
            extras.append(backdata)
        if 'Medium' in item['types']:
            discartdata = {}
            discartdata['image'] = item['image']
            discartdata['preview'] = item['thumbnails']['small']
            discartdata['aspect'] = 'discart'
            extras.append(discartdata)
        # exculde spine+back images
        if 'Spine' in item['types'] and len(item['types']) == 1:
            spinedata = {}
            spinedata['image'] = item['image']
            spinedata['preview'] = item['thumbnails']['small']
            spinedata['aspect'] = 'spine'
            extras.append(spinedata)
    if thumbs:
        albumdata['thumb'] = thumbs
    if extras:
        albumdata['extras'] = extras
    return albumdata



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

import xbmcplugin,xbmcgui,xbmc,xbmcaddon
import os,sys,urllib

def get_params():
        param=[]
        paramstring=sys.argv[2]
        if len(paramstring)>=2:
                params=sys.argv[2]
                cleanedparams=params.replace('?','')
                if (params[len(params)-1]=='/'):
                        params=params[0:len(params)-2]
                pairsofparams=cleanedparams.split('&')
                param={}
                for i in range(len(pairsofparams)):
                        splitparams={}
                        splitparams=pairsofparams[i].split('=')
                        if (len(splitparams))==2:
                                param[splitparams[0]]=splitparams[1]

        return param


params=get_params()

action=urllib.unquote_plus(params["action"])

if action == 'find':
    year = 0
    title=urllib.unquote_plus(params["title"])
    try:
        year=int(urllib.unquote_plus(params["year"]))
    except:
        pass

    print('Find movie with title %s from year %i' %(title, int(year)))
    liz=xbmcgui.ListItem('Demo movie 1', thumbnailImage='DefaultVideo.png', offscreen=True)
    liz.setProperty('relevance', '0.5')
    xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]), url="/path/to/movie", listitem=liz, isFolder=True)
    liz=xbmcgui.ListItem('Demo movie 2', thumbnailImage='DefaultVideo.png', offscreen=True)
    liz.setProperty('relevance', '0.3')
    xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]), url="/path/to/movie2", listitem=liz, isFolder=True)
elif action == 'getdetails':
    url=urllib.unquote_plus(params["url"])
    if url == '/path/to/movie':
        liz=xbmcgui.ListItem('Demo movie 1', offscreen=True)
        liz.setInfo('video',
                    {'title': 'Demo movie 1',
                     'originaltitle': 'Demo måvie 1',
                     'sorttitle': '2',
                     'userrating': 5,
                     'top250': 3,
                     'plotoutline': 'Outline yo',
                     'plot': 'Plot yo',
                     'tagline': 'Tag yo',
                     'duration': 110,
                     'mpaa': 'T',
                     'trailer': '/home/akva/bunnies/unicorns.mkv',
                     'genre': ['Action', 'Comedy'],
                     'country': ['Norway', 'Sweden', 'China'],
                     'credits': ['None', 'Want', 'To Admit It'],
                     'director': ['spiff', 'spiff2'],
                     'set': 'Spiffy creations',
                     'setoverview': 'Horrors created by spiff',
                     'studio': ['Studio1', 'Studio2'],
                     'dateadded': '2016-01-01',
                     'premiered': '2015-01-01',
                     'showlink': ['Demo show 1']
                    })
                    #todo: missing actor thumb aspect
        liz.setRating("imdb", 9, 100000, True )
        liz.setRating("themoviedb", 8.9, 1000)
        liz.setUniqueIDs({ 'imdb': 'tt8938399', 'tmdb' : '9837493' }, 'imdb')
        liz.setCast([{'name': 'spiff', 'role': 'himself', 'thumbnail': '/home/akva/Pictures/fish.jpg', 'order': 2},
                    {'name': 'monkey', 'role': 'orange', 'thumbnail': '/home/akva/Pictures/coffee.jpg', 'order': 1}])
        liz.addAvailableArtwork('DefaultBackFanart.png', 'banner')
        liz.addAvailableArtwork('/home/akva/Pictures/hawaii-shirt.png', 'poster')
        liz.setAvailableFanart([{'image': 'DefaultBackFanart.png', 'preview': 'DefaultBackFanart.png'}, 
                                {'image': '/home/akva/Pictures/hawaii-shirt.png', 'preview': '/home/akva/Pictures/hawaii-shirt.png'}])
        xbmcplugin.setResolvedUrl(handle=int(sys.argv[1]), succeeded=True, listitem=liz)
elif action == 'getartwork':
    url=urllib.unquote_plus(params["id"])
    if url == '456':
        liz=xbmcgui.ListItem('Demo movie 1', offscreen=True)
        liz.addAvailableArtwork('DefaultBackFanart.png', 'banner')
        liz.addAvailableArtwork('/home/akva/Pictures/hawaii-shirt.png', 'poster')
        liz.setAvailableFanart([{'image': 'DefaultBackFanart.png', 'preview': 'DefaultBackFanart.png'}, 
                                {'image': '/home/akva/Pictures/hawaii-shirt.png', 'preview': '/home/akva/Pictures/hawaii-shirt.png'}])
        xbmcplugin.setResolvedUrl(handle=int(sys.argv[1]), succeeded=True, listitem=liz)
elif action == 'nfourl':
    nfo=urllib.unquote_plus(params["nfo"])
    print 'Find url from nfo file'
    liz=xbmcgui.ListItem('Demo movie 1', offscreen=True)
    xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]), url="/path/to/movie1", listitem=liz, isFolder=True)

xbmcplugin.endOfDirectory(int(sys.argv[1]))



# -*- coding: UTF-8 -*-
#
# Copyright (C) 2020, Team Kodi
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# Code attribution:
# core based on metadata.tvmaze scrapper by Roman Miroshnychenko aka Roman V.M.
# IMDb ratings based on code in metadata.themoviedb.org.python by Team Kodi
# pylint: disable=missing-docstring

from __future__ import absolute_import

import sys

from libs.actions import router
from libs.debugger import debug_exception

if __name__ == '__main__':
    with debug_exception():
        router(sys.argv[2][1:])



# -*- coding: utf-8 -*-

"""

    Copyright (C) 2013-2014 Team-XBMC
    Copyright (C) 2014-2019 Team Kodi

    This file is part of service.xbmc.versioncheck

    SPDX-License-Identifier: GPL-3.0-or-later
    See LICENSES/GPL-3.0-or-later.txt for more information.

"""

from .common import log


def compare_version(version_installed, version_list):
    """ Compare the installed version against the provided version list

    :param version_installed: currently installed version
    :type version_installed: dict
    :param version_list: provided versions to compare against
    :type version_list: dict
    :return: old, current, available, and stable versions
    :rtype: bool / 'stable', dict, dict, dict
    """
    # Create separate version lists
    version_list_stable = version_list['releases']['stable']
    version_list_rc = version_list['releases']['releasecandidate']
    version_list_beta = version_list['releases']['beta']
    version_list_alpha = version_list['releases']['alpha']
    # version_list_prealpha = version_list['releases']['prealpha']

    stable_version = version_list_stable[0]
    rc_version = version_list_rc[0]
    beta_version = version_list_beta[0]
    alpha_version = version_list_alpha[0]

    log('Version installed %s' % version_installed)

    # Check to upgrade to newest available stable version
    # check on smaller major version. Smaller version than available always notify

    # check for stable versions
    old_version, version_available = _check_for_stable_version(version_installed, stable_version)

    if not old_version:
        # Already skipped a possible newer stable build. Let's continue with non stable builds.
        # Check also 'old version' hasn't been set to 'stable' or true by previous checks because
        # if so, those part need to be skipped
        old_version, version_available = _check_for_rc_version(version_installed,
                                                               rc_version, beta_version)

    if not old_version:
        # check for beta builds
        old_version, version_available = _check_for_beta_version(version_installed, beta_version)

    if not old_version:
        # check for alpha builds
        old_version, version_available = _check_for_alpha_version(version_installed, alpha_version)

    return old_version, version_installed, version_available, stable_version


def _check_for_stable_version(version_installed, stable_version):
    """ Compare the installed version against the latest stable version

    :param version_installed: currently installed version
    :type version_installed: dict
    :param stable_version: latest stable version
    :type stable_version: dict
    :return: whether using an old version, and available version if newer stable version available
    :rtype: bool / 'stable', dict
    """
    # check if installed major version is smaller than available major stable
    # here we don't care if running non stable
    old_version = False
    version_available = {}

    if version_installed['major'] < int(stable_version['major']):
        version_available = stable_version
        old_version = 'stable'
        log('Version available  %s' % stable_version)
        log('You are running an older version')

    # check if installed major version is equal than available major stable
    # however also check on minor version and still don't care about non stable
    elif version_installed['major'] == int(stable_version['major']):
        if version_installed['minor'] < int(stable_version['minor']):
            version_available = stable_version
            old_version = 'stable'
            log('Version available  %s' % stable_version)
            log('You are running an older minor version')
        # check for <= minor !stable
        elif version_installed['tag'] != 'stable' and \
                version_installed['minor'] <= int(stable_version['minor']):
            version_available = stable_version
            old_version = True
            log('Version available  %s' % stable_version)
            log('You are running an older non stable minor version')
        else:
            log('Version available  %s' % stable_version)
            log('There is no newer stable available')

    return old_version, version_available


def _check_for_rc_version(version_installed, rc_version, beta_version):
    """ Compare the installed version against the latest RC version

    :param version_installed: currently installed version
    :type version_installed: dict
    :param rc_version: latest rc version
    :type rc_version: dict
    :param beta_version: latest beta version
    :type beta_version: dict
    :return: whether using an old version, and available version if newer rc version available
    :rtype: bool, dict
    """
    old_version = False
    version_available = {}
    # check for RC builds
    if version_installed['tag'] in ['releasecandidate']:
        # check if you are using a RC build lower than current available RC
        # then check if you are using a beta/alpha lower than current available RC
        # 14.0rc3 is newer than:  14.0rc1, 14.0b9, 14.0a15
        if version_installed['major'] <= int(rc_version['major']):
            if version_installed['minor'] <= int(rc_version['minor']):
                if version_installed.get('tagversion', '') < rc_version['tagversion']:
                    version_available = rc_version
                    old_version = True
                    log('Version available  %s' % rc_version)
                    log('You are running an older RC version')
    # now check if installed !=rc
    elif version_installed['tag'] in ['beta', 'alpha', 'prealpha']:
        if version_installed['major'] <= int(rc_version['major']):
            if version_installed['minor'] <= int(beta_version['minor']):
                version_available = rc_version
                old_version = True
                log('Version available  %s' % rc_version)
                log('You are running an older non RC version')

    return old_version, version_available


def _check_for_beta_version(version_installed, beta_version):
    """ Compare the installed version against the latest beta version

    :param version_installed: currently installed version
    :type version_installed: dict
    :param beta_version: latest beta version
    :type beta_version: dict
    :return: whether using an old version, and available version if newer beta version available
    :rtype: bool, dict
    """
    old_version = False
    version_available = {}
    # check for beta builds
    if not old_version and version_installed['tag'] == 'beta':
        # check if you are using a RC build lower than current available RC
        # then check if you are using a beta/alpha lower than current available RC
        # 14.0b3 is newer than:  14.0b1, 14.0a15
        if version_installed['major'] <= int(beta_version['major']):
            if version_installed['minor'] <= int(beta_version['minor']):
                if version_installed.get('tagversion', '') < beta_version['tagversion']:
                    version_available = beta_version
                    old_version = True
                    log('Version available  %s' % beta_version)
                    log('You are running an older beta version')
    # now check if installed !=beta
    elif not old_version and version_installed['tag'] in ['alpha', 'prealpha']:
        if version_installed['major'] <= int(beta_version['major']):
            if version_installed['minor'] <= int(beta_version['minor']):
                version_available = beta_version
                old_version = True
                log('Version available  %s' % beta_version)
                log('You are running an older non beta version')

    return old_version, version_available


def _check_for_alpha_version(version_installed, alpha_version):
    """ Compare the installed version against the latest alpha version

    :param version_installed: currently installed version
    :type version_installed: dict
    :param alpha_version: latest alpha version
    :type alpha_version: dict
    :return: whether using an old version, and available version if newer alpha version available
    :rtype: bool, dict
    """
    old_version = False
    version_available = {}
    # check for alpha builds and older
    if version_installed['tag'] == 'alpha':
        # check if you are using a RC build lower than current available RC
        # then check if you are using a beta/alpha lower than current available RC
        # 14.0a3 is newer than: 14.0a1 or pre-alpha
        if version_installed['major'] <= int(alpha_version['major']):
            if version_installed['minor'] <= int(alpha_version['minor']):
                if version_installed.get('tagversion', '') < alpha_version['tagversion']:
                    version_available = alpha_version
                    old_version = True
                    log('Version available  %s' % alpha_version)
                    log('You are running an older alpha version')
    # now check if installed !=alpha
    elif version_installed['tag'] in ['prealpha']:
        if version_installed['major'] <= int(alpha_version['major']):
            if version_installed['minor'] <= int(alpha_version['minor']):
                version_available = alpha_version
                old_version = True
                log('Version available  %s' % alpha_version)
                log('You are running an older non alpha version')

    return old_version, version_available



# -*- coding: UTF-8 -*-
#
# Copyright (C) 2020, Team Kodi
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# IMDb ratings based on code in metadata.themoviedb.org.python by Team Kodi
# pylint: disable=missing-docstring

import re
from . import api_utils
from . import get_imdb_id

IMDB_RATINGS_URL = 'https://www.imdb.com/title/{}/'
IMDB_RATING_REGEX = re.compile(r'itemprop="ratingValue".*?>.*?([\d.]+).*?<')
IMDB_VOTES_REGEX = re.compile(r'itemprop="ratingCount".*?>.*?([\d,]+).*?<')
IMDB_TOP250_REGEX = re.compile(r'Top Rated Movies #(\d+)')

def get_details(uniqueids):
    imdb_id = get_imdb_id(uniqueids)
    if not imdb_id:
        return {}
    votes, rating, top250 = _get_ratinginfo(imdb_id)
    return _assemble_imdb_result(votes, rating, top250)

def _get_ratinginfo(imdb_id):
    response = api_utils.load_info(IMDB_RATINGS_URL.format(imdb_id), default = '', resp_type='text')
    return _parse_imdb_result(response)

def _assemble_imdb_result(votes, rating, top250):
    result = {}
    if top250:
        result['info'] = {'top250': top250}
    if votes and rating:
        result['ratings'] = {'imdb': {'votes': votes, 'rating': rating}}
    return result

def _parse_imdb_result(input_html):
    rating = _parse_imdb_rating(input_html)
    votes = _parse_imdb_votes(input_html)
    top250 = _parse_imdb_top250(input_html)

    return votes, rating, top250

def _parse_imdb_rating(input_html):
    match = re.search(IMDB_RATING_REGEX, input_html)
    if (match):
        return float(match.group(1))
    return None

def _parse_imdb_votes(input_html):
    match = re.search(IMDB_VOTES_REGEX, input_html)
    if (match):
        return int(match.group(1).replace(',', ''))
    return None

def _parse_imdb_top250(input_html):
    match = re.search(IMDB_TOP250_REGEX, input_html)
    if (match):
        return int(match.group(1))
    return None



#! /usr/bin/python

#############################################################
# This tool is used to generate the Platinum documentation  #
#############################################################

import os
import sys
import subprocess
import shutil
import zipfile
import tarfile

#############################################################
# ZIP support
#############################################################
def ZipDir(top, archive, dir) :
    entries = os.listdir(top)
    for entry in entries: 
        path = os.path.join(top, entry)
        if os.path.isdir(path):
            ZipDir(path, archive, os.path.join(dir, entry))
        else:
            zip_name = os.path.join(dir, entry)
            archive.write(path, zip_name)

def ZipIt(root, dir) :
    zip_filename = root+'/'+dir+'.zip'
   
    if os.path.exists(zip_filename):
        os.remove(zip_filename)

    archive = zipfile.ZipFile(zip_filename, "w", zipfile.ZIP_DEFLATED)
    ZipDir(root+'/'+dir, archive, dir)
    archive.close()

def TarIt(root, dir) :
    tar_filename = root+'/'+dir+'.tgz'
   
    if os.path.exists(tar_filename):
        os.remove(tar_filename)

    archive = tarfile.TarFileCompat(tar_filename, "w", tarfile.TAR_GZIPPED)
    ZipDir(root+'/'+dir, archive, dir)
    archive.close()

#############################################################
# Main
#############################################################
# ensure that PLATINUM_KIT_HOME has been set and exists
if not os.environ.has_key('PLATINUM_KIT_HOME'):
    print 'ERROR: PLATINUM_KIT_HOME not set'
    sys.exit(1)
PLATINUM_KIT_HOME = os.environ['PLATINUM_KIT_HOME']
    
if not os.path.exists(PLATINUM_KIT_HOME) :
    print 'ERROR: PLATINUM_KIT_HOME ('+PLATINUM_KIT_HOME+') does not exist'
    sys.exit(1)
else :
    print 'PLATINUM_KIT_HOME = ' + PLATINUM_KIT_HOME
    
# compute paths
SDK_DOC_NAME='Platinum-HTML'
SDK_DOC_ROOT=PLATINUM_KIT_HOME+'/Platinum/Docs/Doxygen'

# start doxygen
retcode = subprocess.call(['doxygen'], cwd=SDK_DOC_ROOT)

if retcode != 0:
    print 'ERROR: doxygen failed'
    sys.exit(1)
    
# zip documentation
ZipIt(SDK_DOC_ROOT, SDK_DOC_NAME)

# cleanup
shutil.rmtree(SDK_DOC_ROOT+'/'+SDK_DOC_NAME)



# -*- coding: utf-8 -*-

"""

    Copyright (C) 2019 Team Kodi

    This file is part of service.xbmc.versioncheck

    SPDX-License-Identifier: GPL-3.0-or-later
    See LICENSES/GPL-3.0-or-later.txt for more information.

"""

from .common import get_password_from_user
from .common import log


class Handler:
    """ Base handler class for apt_daemon_handler, and shell_handler_apt
    """

    def __init__(self):
        self._pwd = ''
        self._update = True

    @property
    def pwd(self):
        """ password property

        :return: password
        :rtype: str
        """
        return self._pwd

    @pwd.setter
    def pwd(self, value):
        """ password setter

        :param value: password
        :type value: str
        """
        self._pwd = value

    @property
    def update(self):
        """ update apt-cache property

        :return: whether to update apt-cache or not when checking for upgrades
        :rtype: bool
        """
        return self._update

    @update.setter
    def update(self, value):
        """ update apt-cache setter

        :param value: whether to update apt-cache or not when checking for upgrades
        :type value: bool
        """
        self._update = value

    def _check_versions(self, package):
        raise NotImplementedError

    def check_upgrade_available(self, package):
        """ Check if package upgrade is available

        :param package: package to check for upgrade availability
        :type package: str
        :return: whether an upgrade exists for the provided package
        :rtype: bool
        """
        installed, candidate = self._check_versions(package)
        if installed and candidate:
            if installed != candidate:
                log('Version installed  %s' % installed)
                log('Version available  %s' % candidate)
                return True
            log('Already on newest version')
            return False

        if not installed:
            log('No installed package found')

        return False

    def _get_password(self):
        """ Get password, ask user for password if not known

        :return: password
        :rtype: str
        """
        if not self.pwd:
            self.pwd = get_password_from_user()
        return self.pwd



from . import api_utils
try:
    from urllib import quote
except ImportError: # py2 / py3
    from urllib.parse import quote

API_KEY = '384afe262ee0962545a752ff340e3ce4'
API_URL = 'https://webservice.fanart.tv/v3/movies/{}'

ARTMAP = {
    'movielogo': 'clearlogo',
    'hdmovielogo': 'clearlogo',
    'hdmovieclearart': 'clearart',
    'movieart': 'clearart',
    'moviedisc': 'discart',
    'moviebanner': 'banner',
    'moviethumb': 'landscape',
    'moviebackground': 'fanart',
    'movieposter': 'poster'
}

def get_details(uniqueids, clientkey, language, set_tmdbid):
    media_id = _get_mediaid(uniqueids)
    if not media_id:
        return {}

    movie_data = _get_data(media_id, clientkey)
    movieset_data = _get_data(set_tmdbid, clientkey)
    if not movie_data and not movieset_data:
        return {}

    movie_art = {}
    movieset_art = {}
    if movie_data:
        movie_art = _parse_data(movie_data, language)
    if movieset_data:
        movieset_art = _parse_data(movieset_data, language)
        movieset_art = {'set.' + key: value for key, value in movieset_art.items()}

    available_art = movie_art
    available_art.update(movieset_art)

    return {'available_art': available_art}

def _get_mediaid(uniqueids):
    for source in ('tmdb', 'imdb', 'unknown'):
        if source in uniqueids:
            return uniqueids[source]

def _get_data(media_id, clientkey):
    headers = {'api-key': API_KEY}
    if clientkey:
        headers['client-key'] = clientkey
    api_utils.set_headers(headers)
    fanarttv_url = API_URL.format(media_id)
    return api_utils.load_info(fanarttv_url, default={})

def _parse_data(data, language):
    result = {}
    for arttype, artlist in data.items():
        if arttype not in ARTMAP:
            continue
        for image in artlist:
            image_lang = _get_imagelanguage(arttype, image)
            if image_lang and image_lang != language:
                continue

            generaltype = ARTMAP[arttype]
            if generaltype == 'poster' and not image_lang:
                generaltype = 'keyart'
            if artlist and generaltype not in result:
                result[generaltype] = []

            url = quote(image['url'], safe="%/:=&?~#+!$,;'@()*[]")
            resultimage = {'url': url, 'preview': url.replace('.fanart.tv/fanart/', '.fanart.tv/preview/')}
            result[generaltype].append(resultimage)

    return result

def _get_imagelanguage(arttype, image):
    if 'lang' not in image or arttype == 'moviebackground':
        return None
    if arttype in ('movielogo', 'hdmovielogo', 'hdmovieclearart', 'movieart', 'moviebanner',
            'moviethumb', 'moviedisc'):
        return image['lang'] if image['lang'] not in ('', '00') else 'en'
    # movieposter may or may not have a title and thus need a language
    return image['lang'] if image['lang'] not in ('', '00') else None



# -*- coding: UTF-8 -*-
#
# Copyright (C) 2020, Team Kodi
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
# pylint: disable=missing-docstring

import json, sys, urllib.parse
from .utils import logger
from . import api_utils
from xbmcaddon import Addon
from datetime import datetime, timedelta


def _get_date_numeric(datetime_):
    return (datetime_ - datetime(1970, 1, 1)).total_seconds()


def _get_configuration():
    logger.debug('getting configuration details')
    return api_utils.load_info('https://api.themoviedb.org/3/configuration', params={'api_key': TMDB_CLOWNCAR}, verboselog=VERBOSELOG)


def _load_base_urls():
    image_root_url = ADDON.getSettingString('originalUrl')
    preview_root_url = ADDON.getSettingString('previewUrl')
    last_updated = ADDON.getSettingString('lastUpdated')
    if not image_root_url or not preview_root_url or not last_updated or \
            float(last_updated) < _get_date_numeric(datetime.now() - timedelta(days=30)):
        conf = _get_configuration()
        if conf:
            image_root_url = conf['images']['secure_base_url'] + 'original'
            preview_root_url = conf['images']['secure_base_url'] + 'w780'
            ADDON.setSetting('originalUrl', image_root_url)
            ADDON.setSetting('previewUrl', preview_root_url)
            ADDON.setSetting('lastUpdated', str(_get_date_numeric(datetime.now())))
    return image_root_url, preview_root_url


ADDON = Addon()
TMDB_CLOWNCAR = 'af3a53eb387d57fc935e9128468b1899'
FANARTTV_CLOWNCAR = 'b018086af0e1478479adfc55634db97d'
TRAKT_CLOWNCAR = '90901c6be3b2de5a4fa0edf9ab5c75e9a5a0fef2b4ee7373d8b63dcf61f95697'
MAXIMAGES = 350
FANARTTV_MAPPING = { 'showbackground': 'backdrops',
                     'tvposter': 'posters',
                     'tvbanner': 'banner',
                     'hdtvlogo': 'clearlogo',
                     'clearlogo': 'clearlogo',
                     'hdclearart': 'clearart',
                     'clearart': 'clearart',
                     'tvthumb': 'landscape',
                     'characterart': 'characterart',
                     'seasonposter':'seasonposters',
                     'seasonbanner':'seasonbanner',
                     'seasonthumb': 'seasonlandscape'
                   }

try:
    source_params = dict(urllib.parse.parse_qsl(sys.argv[2]))
except IndexError:
    source_params = {}
source_settings = json.loads(source_params.get('pathSettings', {}))

KEEPTITLE =source_settings.get('keeporiginaltitle', ADDON.getSettingBool('keeporiginaltitle'))
CATLANDSCAPE = source_settings.get('cat_landscape', True)
VERBOSELOG =  source_settings.get('verboselog', ADDON.getSettingBool('verboselog'))
LANG = source_settings.get('language', ADDON.getSettingString('language'))
CERT_COUNTRY = source_settings.get('tmdbcertcountry', ADDON.getSettingString('tmdbcertcountry')).lower()
IMAGEROOTURL, PREVIEWROOTURL = _load_base_urls()

if source_settings.get('usecertprefix', ADDON.getSettingBool('usecertprefix')):
    CERT_PREFIX = source_settings.get('certprefix', ADDON.getSettingString('certprefix'))
else:
    CERT_PREFIX = ''
primary_rating = source_settings.get('ratings', ADDON.getSettingString('ratings')).lower()
RATING_TYPES = [primary_rating]
if source_settings.get('imdbanyway', ADDON.getSettingBool('imdbanyway')) and primary_rating != 'imdb':
    RATING_TYPES.append('imdb')
if source_settings.get('traktanyway', ADDON.getSettingBool('traktanyway')) and primary_rating != 'trakt':
    RATING_TYPES.append('trakt')
if source_settings.get('tmdbanyway', ADDON.getSettingBool('tmdbanyway')) and primary_rating != 'tmdb':
    RATING_TYPES.append('tmdb')
FANARTTV_ENABLE = source_settings.get('enable_fanarttv', ADDON.getSettingBool('enable_fanarttv'))
FANARTTV_CLIENTKEY = source_settings.get('fanarttv_clientkey', ADDON.getSettingString('fanarttv_clientkey'))



# -*- coding: utf-8 -*-

AUDIODBKEY = '95424d43204d6564696538'
AUDIODBURL = 'https://www.theaudiodb.com/api/v1/json/%s/%s'
AUDIODBSEARCH = 'searchalbum.php?s=%s&a=%s'
AUDIODBDETAILS = 'album-mb.php?i=%s'

MUSICBRAINZURL = 'https://musicbrainz.org/ws/2/%s'
MUSICBRAINZSEARCH = 'release/?query=release:"%s"%%20AND%%20(artistname:"%s"%%20OR%%20artist:"%s")&fmt=json'
MUSICBRAINZLINKS = 'release-group/%s?inc=url-rels&fmt=json'
MUSICBRAINZDETAILS = 'release/%s?inc=release-groups+artists+labels+ratings&fmt=json'
MUSICBRAINZART = 'https://coverartarchive.org/release-group/%s'

DISCOGSKEY = 'zACPgktOmNegwbwKWMaC'
DISCOGSSECRET = 'wGuSOeMtfdkQxtERKQKPquyBwExSHdQq'
DISCOGSURL = 'https://api.discogs.com/%s'
DISCOGSSEARCH = 'database/search?release_title=%s&type=release&artist=%s&page=1&per_page=100&key=%s&secret=%s'
DISCOGSMASTER = 'masters/%s?key=%s&secret=%s'
DISCOGSDETAILS = 'releases/%s?key=%s&secret=%s'

ALLMUSICURL = 'https://www.allmusic.com/%s'
ALLMUSICSEARCH = 'search/albums/%s+%s'
ALLMUSICDETAILS = '%s/releases'

FANARTVKEY = '88ca41db0d6878929f1f9771eade41fd'
FANARTVURL = 'https://webservice.fanart.tv/v3/music/albums/%s?api_key=%s'

WIKIDATAURL = 'https://www.wikidata.org/wiki/Special:EntityData/%s.json'
WIKIPEDIAURL = 'https://en.wikipedia.org/w/api.php?action=query&format=json&prop=extracts&titles=%s&formatversion=2&exsentences=10&exlimit=1&explaintext=1'



# -*- coding: utf-8 -*-

def theaudiodb_albumdetails(data):
    if data.get('album'):
        item = data['album'][0]
        albumdata = {}
        albumdata['album'] = item['strAlbum']
        if item.get('intYearReleased',''):
            albumdata['year'] = item['intYearReleased']
        if item.get('strStyle',''):
            albumdata['styles'] = item['strStyle']
        if item.get('strGenre',''):
            albumdata['genre'] = item['strGenre']
        if item.get('strLabel',''):
            albumdata['label'] = item['strLabel']
        if item.get('strReleaseFormat',''):
            albumdata['type'] = item['strReleaseFormat']
        if item.get('intScore',''):
            albumdata['rating'] = str(int(float(item['intScore']) + 0.5))
        if item.get('intScoreVotes',''):
            albumdata['votes'] = item['intScoreVotes']
        if item.get('strMood',''):
            albumdata['moods'] = item['strMood']
        if item.get('strTheme',''):
            albumdata['themes'] = item['strTheme']
        if item.get('strMusicBrainzID',''):
            albumdata['mbreleasegroupid'] = item['strMusicBrainzID']
        # api inconsistent
        if item.get('strDescription',''):
            albumdata['descriptionEN'] = item['strDescription']
        elif item.get('strDescriptionEN',''):
            albumdata['descriptionEN'] = item['strDescriptionEN']
        if item.get('strDescriptionDE',''):
            albumdata['descriptionDE'] = item['strDescriptionDE']
        if item.get('strDescriptionFR',''):
            albumdata['descriptionFR'] = item['strDescriptionFR']
        if item.get('strDescriptionCN',''):
            albumdata['descriptionCN'] = item['strDescriptionCN']
        if item.get('strDescriptionIT',''):
            albumdata['descriptionIT'] = item['strDescriptionIT']
        if item.get('strDescriptionJP',''):
            albumdata['descriptionJP'] = item['strDescriptionJP']
        if item.get('strDescriptionRU',''):
            albumdata['descriptionRU'] = item['strDescriptionRU']
        if item.get('strDescriptionES',''):
            albumdata['descriptionES'] = item['strDescriptionES']
        if item.get('strDescriptionPT',''):
            albumdata['descriptionPT'] = item['strDescriptionPT']
        if item.get('strDescriptionSE',''):
            albumdata['descriptionSE'] = item['strDescriptionSE']
        if item.get('strDescriptionNL',''):
            albumdata['descriptionNL'] = item['strDescriptionNL']
        if item.get('strDescriptionHU',''):
            albumdata['descriptionHU'] = item['strDescriptionHU']
        if item.get('strDescriptionNO',''):
            albumdata['descriptionNO'] = item['strDescriptionNO']
        if item.get('strDescriptionIL',''):
            albumdata['descriptionIL'] = item['strDescriptionIL']
        if item.get('strDescriptionPL',''):
            albumdata['descriptionPL'] = item['strDescriptionPL']
        if item.get('strArtist',''):
            albumdata['artist_description'] = item['strArtist']
            artists = []
            artistdata = {}
            artistdata['artist'] = item['strArtist']
            if item.get('strMusicBrainzArtistID',''):
                artistdata['mbartistid'] = item['strMusicBrainzArtistID']
            artists.append(artistdata)
            albumdata['artist'] = artists
        thumbs = []
        extras = []
        if item.get('strAlbumThumb',''):
            thumbdata = {}
            thumbdata['image'] = item['strAlbumThumb']
            thumbdata['preview'] = item['strAlbumThumb'] + '/preview'
            thumbdata['aspect'] = 'thumb'
            thumbs.append(thumbdata)
        if item.get('strAlbumThumbBack',''):
            extradata = {}
            extradata['image'] = item['strAlbumThumbBack']
            extradata['preview'] = item['strAlbumThumbBack'] + '/preview'
            extradata['aspect'] = 'back'
            extras.append(extradata)
        if item.get('strAlbumSpine',''):
            extradata = {}
            extradata['image'] = item['strAlbumSpine']
            extradata['preview'] = item['strAlbumSpine'] + '/preview'
            extradata['aspect'] = 'spine'
            extras.append(extradata)
        if item.get('strAlbumCDart',''):
            extradata = {}
            extradata['image'] = item['strAlbumCDart']
            extradata['preview'] = item['strAlbumCDart'] + '/preview'
            extradata['aspect'] = 'discart'
            extras.append(extradata)
        if item.get('strAlbum3DCase',''):
            extradata = {}
            extradata['image'] = item['strAlbum3DCase']
            extradata['preview'] = item['strAlbum3DCase'] + '/preview'
            extradata['aspect'] = '3dcase'
            extras.append(extradata)
        if item.get('strAlbum3DFlat',''):
            extradata = {}
            extradata['image'] = item['strAlbum3DFlat']
            extradata['preview'] = item['strAlbum3DFlat'] + '/preview'
            extradata['aspect'] = '3dflat'
            extras.append(extradata)
        if item.get('strAlbum3DFace',''):
            extradata = {}
            extradata['image'] = item['strAlbum3DFace']
            extradata['preview'] = item['strAlbum3DFace'] + '/preview'
            extradata['aspect'] = '3dface'
            extras.append(extradata)
        if thumbs:
            albumdata['thumb'] = thumbs
        if extras:
            albumdata['extras'] = extras
        return albumdata



# -*- coding: utf-8 -*-

AUDIODBKEY = '95424d43204d6564696538'
AUDIODBURL = 'https://www.theaudiodb.com/api/v1/json/%s/%s'
AUDIODBSEARCH = 'search.php?s=%s'
AUDIODBDETAILS = 'artist-mb.php?i=%s'
AUDIODBDISCOGRAPHY = 'discography-mb.php?s=%s'

MUSICBRAINZURL = 'https://musicbrainz.org/ws/2/artist/%s'
MUSICBRAINZSEARCH = '?query="%s"&fmt=json'
MUSICBRAINZDETAILS = '%s?inc=url-rels+release-groups&type=album&fmt=json'

DISCOGSKEY = 'zACPgktOmNegwbwKWMaC'
DISCOGSSECRET = 'wGuSOeMtfdkQxtERKQKPquyBwExSHdQq'
DISCOGSURL = 'https://api.discogs.com/%s'
DISCOGSSEARCH = 'database/search?q=%s&type=artist&key=%s&secret=%s'
DISCOGSDETAILS = 'artists/%s?key=%s&secret=%s'
DISCOGSDISCOGRAPHY = 'artists/%s/releases?sort=format&page=1&per_page=100&key=%s&secret=%s'

ALLMUSICURL = 'https://www.allmusic.com/search/artists/%s'

FANARTVKEY = '88ca41db0d6878929f1f9771eade41fd'
FANARTVURL = 'https://webservice.fanart.tv/v3/music/%s?api_key=%s'

WIKIDATAURL = 'https://www.wikidata.org/wiki/Special:EntityData/%s.json'
WIKIPEDIAURL = 'https://en.wikipedia.org/w/api.php?action=query&format=json&prop=extracts&titles=%s&formatversion=2&exsentences=10&exlimit=1&explaintext=1'



#!/usr/bin/env python3
#
# XBMC Media Center
# XBMC Send
# Copyright (c) 2009 team-xbmc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import sys, os
import getopt
from socket import *
from time import sleep
try:
    from kodi.xbmcclient import *
except:
    sys.path.append(os.path.join(os.path.realpath(os.path.dirname(__file__)), '../../lib/python'))
    from xbmcclient import *

TYPE_ACTION = 'action'
TYPE_BUTTON = 'button'
TYPE_DELAY = 'delay'

def usage():
    print("Usage")
    print("\tkodi-send [OPTION] --action=ACTION")
    print("\tkodi-send [OPTION] --button=BUTTON")
    print('Example')
    print('\tkodi-send --host=192.168.0.1 --port=9777 --action="Quit"')
    print("Options")
    print("\t-?, --help\t\t\tWill bring up this message")
    print("\t--host=HOST\t\t\tChoose what HOST to connect to (default=localhost)")
    print("\t--port=PORT\t\t\tChoose what PORT to connect to (default=9777)")
    print("\t--keymap=KEYMAP\t\t\tChoose which KEYMAP to use for key presses (default=KB)")
    print('\t--button=BUTTON\t\t\tSends a key press event to Kodi, this option can be added multiple times to create a macro')
    print('\t-a ACTION, --action=ACTION\tSends an action to XBMC, this option can be added multiple times to create a macro')
    print('\t-d T, --delay=T\t\t\tWaits for T ms, this option can be added multiple times to create a macro')
    pass

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?pa:d:v", ["help", "host=", "port=", "keymap=", "button=", "action=", "delay="])
    except getopt.GetoptError as err:
        # print help information and exit:
        print(str(err)) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    ip = "localhost"
    port = 9777
    keymap = "KB"
    actions = []
    verbose = False
    for o, a in opts:
        if o in ("-?", "--help"):
            usage()
            sys.exit()
        elif o == "--host":
            ip = a
        elif o == "--port":
            port = int(a)
        elif o == "--keymap":
            keymap = a
        elif o == "--button":
            actions.append({'type': TYPE_BUTTON, 'content': a})
        elif o in ("-a", "--action"):
            actions.append({'type': TYPE_ACTION, 'content': a})
        elif o in ("-d", "--delay"):
            actions.append({'type': TYPE_DELAY, 'content': int(a)})
        else:
            assert False, "unhandled option"

    addr = (ip, port)
    sock = socket(AF_INET,SOCK_DGRAM)

    if len(actions) == 0:
        usage()
        sys.exit(0)

    for action in actions:
        print('Sending: %s' % action)
        if action['type'] == TYPE_ACTION:
            packet = PacketACTION(actionmessage=action['content'], actiontype=ACTION_BUTTON)
        elif action['type'] == TYPE_BUTTON:
            packet = PacketBUTTON(code=0, repeat=0, down=1, queue=1, map_name=keymap, button_name=action['content'], amount=0)
        elif action['type'] == TYPE_DELAY:
            time.sleep(action['content'] / 1000.0)
            continue
        packet.send(sock, addr, uid=0)

if __name__=="__main__":
    main()



# -*- coding: UTF-8 -*-
#
# Copyright (C) 2020, Team Kodi
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
# pylint: disable=missing-docstring
#
# This is based on the metadata.tvmaze scrapper by Roman Miroshnychenko aka Roman V.M.

"""Functions to process data"""

from __future__ import absolute_import, unicode_literals

import re, json
from collections import OrderedDict, namedtuple
from .utils import safe_get, logger
from . import settings

try:
    from typing import Optional, Text, Dict, List, Any  # pylint: disable=unused-import
    from xbmcgui import ListItem  # pylint: disable=unused-import
    InfoType = Dict[Text, Any]  # pylint: disable=invalid-name
except ImportError:
    pass

TAG_RE = re.compile(r'<[^>]+>')
SHOW_ID_REGEXPS = (
    r'(tvmaze)\.com/shows/(\d+)/[\w\-]',
    r'(thetvdb)\.com/.*?series/(\d+)',
    r'(thetvdb)\.com[\w=&\?/]+id=(\d+)',
    r'(imdb)\.com/[\w/\-]+/(tt\d+)',
    r'(themoviedb)\.org/tv/(\d+).*/episode_group/(.*)',
    r'(themoviedb)\.org/tv/(\d+)',
    r'(themoviedb)\.org/./tv/(\d+)',
    r'(tmdb)\.org/./tv/(\d+)'
)
SUPPORTED_ARTWORK_TYPES = {'poster', 'banner'}
IMAGE_SIZES = ('large', 'original', 'medium')
CLEAN_PLOT_REPLACEMENTS = (
    ('<b>', '[B]'),
    ('</b>', '[/B]'),
    ('<i>', '[I]'),
    ('</i>', '[/I]'),
    ('</p><p>', '[CR]'),
)
VALIDEXTIDS = ['tmdb_id', 'imdb_id', 'tvdb_id']

UrlParseResult = namedtuple('UrlParseResult', ['provider', 'show_id', 'ep_grouping'])



def _clean_plot(plot):
    # type: (Text) -> Text
    """Replace HTML tags with Kodi skin tags"""
    for repl in CLEAN_PLOT_REPLACEMENTS:
        plot = plot.replace(repl[0], repl[1])
    plot = TAG_RE.sub('', plot)
    return plot


def _set_cast(cast_info, list_item):
    # type: (InfoType, ListItem) -> ListItem
    """Save cast info to list item"""
    cast = []
    for item in cast_info:
        data = {
            'name': item['name'],
            'role': item.get('character', item.get('character_name', '')),
            'order': item['order'],
        }
        thumb = None
        if safe_get(item, 'profile_path') is not None:
            thumb = settings.IMAGEROOTURL + item['profile_path']
        if thumb:
            data['thumbnail'] = thumb
        cast.append(data)
    list_item.setCast(cast)
    return list_item


def _get_credits(show_info):
    # type: (InfoType) -> List[Text]
    """Extract show creator(s) and writer(s) from show info"""
    credits = []
    for item in show_info.get('created_by', []):
        credits.append(item['name'])
    for item in show_info.get('credits', {}).get('crew', []):
        isWriter = item.get('job', '').lower() == 'writer' or item.get('department', '').lower() == 'writing'
        if isWriter and item.get('name') not in credits:
            credits.append(item['name'])
    return credits


def _get_directors(episode_info):
    # type: (InfoType) -> List[Text]
    """Extract episode writer(s) from episode info"""
    directors_ = []
    for item in episode_info.get('credits', {}).get('crew', []):
        if item.get('job') == 'Director':
            directors_.append(item['name'])
    return directors_


def _set_unique_ids(ext_ids, list_item):
    # type: (InfoType, ListItem) -> ListItem
    """Extract unique ID in various online databases"""
    unique_ids = {}
    for key, value in ext_ids.items():
        if key in VALIDEXTIDS and value:
            key = key[:-3]
            unique_ids[key] = str(value)
    list_item.setUniqueIDs(unique_ids, 'tmdb')
    return list_item


def _set_rating(the_info, list_item, episode=False):
    # type: (InfoType, ListItem) -> ListItem
    """Set show/episode rating"""
    first = True
    for rating_type in settings.RATING_TYPES:
        logger.debug('adding rating type of %s' % rating_type)
        rating = float(the_info.get('ratings', {}).get(rating_type, {}).get('rating', '0'))
        votes = int(the_info.get('ratings', {}).get(rating_type, {}).get('votes', '0'))
        logger.debug("adding rating of %s and votes of %s" % (str(rating), str(votes)))
        if rating > 0:
            list_item.setRating(rating_type, rating, votes=votes, defaultt=first)
            first = False
    return list_item


def _add_season_info(show_info, list_item):
    # type: (InfoType, ListItem) -> ListItem
    """Add info for show seasons"""
    for season in show_info['seasons']:
        logger.debug('adding information for season %s to list item' % season['season_number'])
        list_item.addSeason(season['season_number'], safe_get(season, 'name', ''))
        for image_type, image_list in season.get('images', {}).items():
            if image_type == 'posters':
                destination = 'poster'
            else:
                destination = image_type
            for image in image_list:
                if image.get('type') == 'fanarttv':
                    theurl = image['file_path']
                    previewurl = theurl.replace('.fanart.tv/fanart/', '.fanart.tv/preview/')
                else:
                    theurl = settings.IMAGEROOTURL + image['file_path']
                    previewurl = settings.PREVIEWROOTURL + image['file_path']
                list_item.addAvailableArtwork(theurl, art_type=destination, preview=previewurl, season=season['season_number'])
    return list_item


def get_image_urls( image ):
    if image.get('type') == 'fanarttv':
        theurl = image['file_path']
        previewurl = theurl.replace('.fanart.tv/fanart/', '.fanart.tv/preview/')
    else:
        theurl = settings.IMAGEROOTURL + image['file_path']
        previewurl = settings.PREVIEWROOTURL + image['file_path']
    return theurl, previewurl


def set_show_artwork(show_info, list_item):
    # type: (InfoType, ListItem) -> ListItem
    """Set available images for a show"""
    for image_type, image_list in show_info.get('images', {}).items():
        if image_type == 'backdrops':
            fanart_list = []
            for image in image_list:
                if image.get('type') == 'fanarttv':
                    theurl = image['file_path']
                else:
                    theurl = settings.IMAGEROOTURL + image['file_path']
                if image.get('iso_639_1') != None and settings.CATLANDSCAPE:
                    theurl, previewurl = get_image_urls( image )
                    list_item.addAvailableArtwork(theurl, art_type="landscape", preview=previewurl)
                else:
                    fanart_list.append({'image': theurl})
            if fanart_list:
                list_item.setAvailableFanart(fanart_list)
        else:
            if image_type == 'posters':
                destination = 'poster'
            else:
                destination = image_type
            for image in image_list:
                theurl, previewurl = get_image_urls( image )
                list_item.addAvailableArtwork(theurl, art_type=destination, preview=previewurl)
    return list_item


def add_main_show_info(list_item, show_info, full_info=True):
    # type: (ListItem, InfoType, bool) -> ListItem
    """Add main show info to a list item"""
    plot = _clean_plot(safe_get(show_info, 'overview', ''))
    original_name = show_info.get('original_name')
    if settings.KEEPTITLE and original_name:
        showname = original_name
    else:
        showname = show_info['name']
    video = {
        'plot': plot,
        'plotoutline': plot,
        'title': showname,
        'originaltitle': original_name,
        'tvshowtitle': showname,
        'mediatype': 'tvshow',
        # This property is passed as "url" parameter to getepisodelist call
        'episodeguide': str(show_info['id']),
    }
    if show_info.get('first_air_date'):
        video['year'] = int(show_info['first_air_date'][:4])
        video['premiered'] = show_info['first_air_date']
    if full_info:
        video['status'] = safe_get(show_info, 'status', '')
        genre_list = safe_get(show_info, 'genres', {})
        genres = []
        for genre in genre_list:
            genres.append(genre['name'])
        video['genre'] = genres
        networks = show_info.get('networks', [])
        if networks:
            network = networks[0]
            country = network.get('origin_country', '')
        else:
            network = None
            country = None
        if network and country:
            video['studio'] = '{0} ({1})'.format(network['name'], country)
            video['country'] = country
        content_ratings = show_info.get('content_ratings', {}).get('results', {})
        if content_ratings:
            mpaa = ''
            mpaa_backup = ''
            for content_rating in content_ratings:
                iso = content_rating.get('iso_3166_1', '').lower()
                if iso == 'us':
                    mpaa_backup = content_rating.get('rating')
                if iso == settings.CERT_COUNTRY.lower():
                    mpaa = content_rating.get('rating', '')
            if not mpaa:
                mpaa = mpaa_backup
            if mpaa:
                video['Mpaa'] = settings.CERT_PREFIX + mpaa
        video['credits'] = video['writer'] = _get_credits(show_info)
        list_item = set_show_artwork(show_info, list_item)
        list_item = _add_season_info(show_info, list_item)
        list_item = _set_cast(show_info['credits']['cast'], list_item)
        list_item = _set_rating(show_info, list_item)
        ext_ids = {'tmdb_id': show_info['id']}
        ext_ids.update(show_info.get('external_ids', {}))
        list_item =  _set_unique_ids(ext_ids, list_item)
    else:
        image = safe_get(show_info, 'poster_path', '')
        if image:
            theurl = settings.IMAGEROOTURL + image
            previewurl = settings.PREVIEWROOTURL + image
            list_item.addAvailableArtwork(theurl, art_type='poster', preview=previewurl)
    logger.debug('adding tv show information for %s to list item' % video['tvshowtitle'])
    list_item.setInfo('video', video)
    # This is needed for getting artwork
    list_item = _set_unique_ids(show_info, list_item)
    return list_item


def add_episode_info(list_item, episode_info, full_info=True):
    # type: (ListItem, InfoType, bool) -> ListItem
    """Add episode info to a list item"""
    video = {
        'title': episode_info.get('name', 'Episode ' + str(episode_info['episode_number'])),
        'season': episode_info['season_number'],
        'episode': episode_info['episode_number'],
        'mediatype': 'episode',
    }
    if safe_get(episode_info, 'air_date') is not None:
        video['aired'] = episode_info['air_date']
    if full_info:
        summary = safe_get(episode_info, 'overview')
        if summary is not None:
            video['plot'] = video['plotoutline'] = _clean_plot(summary)
        if safe_get(episode_info, 'air_date') is not None:
            video['premiered'] = episode_info['air_date']
        list_item = _set_cast(episode_info['credits']['guest_stars'], list_item)
        ext_ids = {'tmdb_id': episode_info['id']}
        ext_ids.update(episode_info.get('external_ids', {}))
        list_item = _set_unique_ids(ext_ids, list_item)
        list_item = _set_rating(episode_info, list_item, episode=True)
        for image in episode_info.get('images', {}).get('stills', []):
            img_path = image.get('file_path')
            if img_path:
                theurl = settings.IMAGEROOTURL + img_path
                previewurl = settings.PREVIEWROOTURL + img_path
                list_item.addAvailableArtwork(theurl, art_type='thumb', preview=previewurl)
        video['credits'] = video['writer'] = _get_credits(episode_info)
        video['director'] = _get_directors(episode_info)
    logger.debug('adding episode information for S%sE%s - %s to list item' % (video['season'], video['episode'], video['title']))
    list_item.setInfo('video', video)
    return list_item


def parse_nfo_url(nfo):
    # type: (Text) -> Optional[UrlParseResult]
    """Extract show ID and named seasons from NFO file contents"""
    # work around for xbmcgui.ListItem.addSeason overwriting named seasons from NFO files
    ns_regex = r'<namedseason number="(.*)">(.*)</namedseason>'
    ns_match = re.findall(ns_regex, nfo, re.I)
    sid_match = None
    for regexp in SHOW_ID_REGEXPS:
        logger.debug('trying regex to match service from parsing nfo:')
        logger.debug(regexp)
        show_id_match = re.search(regexp, nfo, re.I)
        if show_id_match:
            logger.debug('match group 1: ' + show_id_match.group(1))
            logger.debug('match group 2: ' + show_id_match.group(2))
            try:
                ep_grouping = show_id_match.group(3)
            except IndexError:
                ep_grouping = None
            if ep_grouping is not None:
                logger.debug('match group 3: ' + ep_grouping)
            else:
                logger.debug('match group 3: None')
            sid_match = UrlParseResult(show_id_match.group(1), show_id_match.group(2), ep_grouping)
            break
    return sid_match, ns_match


def parse_media_id(title):
    title = title.lower()
    if title.startswith('tt') and title[2:].isdigit():
        return {'type': 'imdb_id', 'title': title} # IMDB ID works alone because it is clear
    elif title.startswith('imdb/tt') and title[7:].isdigit(): # IMDB ID with prefix to match
        return {'type': 'imdb_id', 'title': title[5:]} # IMDB ID works alone because it is clear
    elif title.startswith('tmdb/') and title[5:].isdigit(): # TVDB ID
        return {'type': 'tmdb_id', 'title': title[5:]}
    elif title.startswith('tvdb/') and title[5:].isdigit(): # TVDB ID
        return {'type': 'tvdb_id', 'title': title[5:]}
    return None



from datetime import datetime, timedelta
from . import tmdbapi


class TMDBMovieScraper(object):
    def __init__(self, url_settings, language, certification_country):
        self.url_settings = url_settings
        self.language = language
        self.certification_country = certification_country
        self._urls = None

    @property
    def urls(self):
        if not self._urls:
            self._urls = _load_base_urls(self.url_settings)
        return self._urls

    def search(self, title, year=None):
        search_media_id = _parse_media_id(title)
        if search_media_id:
            if search_media_id['type'] == 'tmdb':
                result = _get_movie(search_media_id['id'], self.language, True)
                result = [result]
            else:
                response = tmdbapi.find_movie_by_external_id(search_media_id['id'], language=self.language)
                theerror = response.get('error')
                if theerror:
                    return 'error: {}'.format(theerror)
                result = response.get('movie_results')
            if 'error' in result:
                return result
        else:
            response = tmdbapi.search_movie(query=title, year=year, language=self.language)
            theerror = response.get('error')
            if theerror:
                return 'error: {}'.format(theerror)
            result = response['results']
        urls = self.urls

        def is_best(item):
            return item['title'].lower() == title and (
                not year or item.get('release_date', '').startswith(year))
        if result and not is_best(result[0]):
            best_first = next((item for item in result if is_best(item)), None)
            if best_first:
                result = [best_first] + [item for item in result if item is not best_first]

        for item in result:
            if item.get('poster_path'):
                item['poster_path'] = urls['preview'] + item['poster_path']
            if item.get('backdrop_path'):
                item['backdrop_path'] = urls['preview'] + item['backdrop_path']
        return result

    def get_details(self, uniqueids):
        media_id = uniqueids.get('tmdb') or uniqueids.get('imdb')
        details = self._gather_details(media_id)
        if not details:
            return None
        if details.get('error'):
            return details
        return self._assemble_details(**details)

    def _gather_details(self, media_id):
        movie = _get_movie(media_id, self.language)
        if not movie or movie.get('error'):
            return movie

        # don't specify language to get English text for fallback
        movie_fallback = _get_movie(media_id)

        collection = _get_moviecollection(movie['belongs_to_collection'].get('id'), self.language) if \
            movie['belongs_to_collection'] else None
        collection_fallback = _get_moviecollection(movie['belongs_to_collection'].get('id')) if \
            movie['belongs_to_collection'] else None

        return {'movie': movie, 'movie_fallback': movie_fallback, 'collection': collection,
            'collection_fallback': collection_fallback}

    def _assemble_details(self, movie, movie_fallback, collection, collection_fallback):
        info = {
            'title': movie['title'],
            'originaltitle': movie['original_title'],
            'plot': movie.get('overview') or movie_fallback.get('overview'),
            'tagline': movie.get('tagline') or movie_fallback.get('tagline'),
            'studio': _get_names(movie['production_companies']),
            'genre': _get_names(movie['genres']),
            'country': _get_names(movie['production_countries']),
            'credits': _get_cast_members(movie['casts'], 'crew', 'Writing', ['Screenplay', 'Writer', 'Author']),
            'director': _get_cast_members(movie['casts'], 'crew', 'Directing', ['Director']),
            'premiered': movie['release_date'],
            'tag': _get_names(movie['keywords']['keywords'])
        }

        if 'countries' in movie['releases']:
            certcountry = self.certification_country.upper()
            for country in movie['releases']['countries']:
                if country['iso_3166_1'] == certcountry and country['certification']:
                    info['mpaa'] = country['certification']
                    break

        trailer = _parse_trailer(movie.get('trailers', {}), movie_fallback.get('trailers', {}))
        if trailer:
            info['trailer'] = trailer
        if collection:
            info['set'] = collection.get('name') or collection_fallback.get('name')
            info['setoverview'] = collection.get('overview') or collection_fallback.get('overview')
        if movie.get('runtime'):
            info['duration'] = movie['runtime'] * 60

        ratings = {'themoviedb': {'rating': float(movie['vote_average']), 'votes': int(movie['vote_count'])}}
        uniqueids = {'tmdb': movie['id'], 'imdb': movie['imdb_id']}
        cast = [{
                'name': actor['name'],
                'role': actor['character'],
                'thumbnail': self.urls['original'] + actor['profile_path']
                    if actor['profile_path'] else "",
                'order': actor['order']
            }
            for actor in movie['casts'].get('cast', [])
        ]
        available_art = _parse_artwork(movie, collection, self.urls, self.language)

        _info = {'set_tmdbid': movie['belongs_to_collection'].get('id')
            if movie['belongs_to_collection'] else None}

        return {'info': info, 'ratings': ratings, 'uniqueids': uniqueids, 'cast': cast,
            'available_art': available_art, '_info': _info}

def _parse_media_id(title):
    if title.startswith('tt') and title[2:].isdigit():
        return {'type': 'imdb', 'id':title} # IMDB ID works alone because it is clear
    title = title.lower()
    if title.startswith('tmdb/') and title[5:].isdigit(): # TMDB ID
        return {'type': 'tmdb', 'id':title[5:]}
    elif title.startswith('imdb/tt') and title[7:].isdigit(): # IMDB ID with prefix to match
        return {'type': 'imdb', 'id':title[5:]}
    return None

def _get_movie(mid, language=None, search=False):
    details = None if search else \
        'trailers,images,releases,casts,keywords' if language is not None else \
        'trailers'
    response = tmdbapi.get_movie(mid, language=language, append_to_response=details)
    theerror = response.get('error')
    if theerror:
        return 'error: {}'.format(theerror)
    else:
        return response

def _get_moviecollection(collection_id, language=None):
    if not collection_id:
        return None
    details = 'images'
    response = tmdbapi.get_collection(collection_id, language=language, append_to_response=details)
    theerror = response.get('error')
    if theerror:
        return 'error: {}'.format(theerror)
    else:
        return response

def _parse_artwork(movie, collection, urlbases, language):
    if language:
        # Image languages don't have regional variants
        language = language.split('-')[0]
    posters = []
    landscape = []
    fanart = []
    if 'images' in movie:
        posters = _get_images_with_fallback(movie['images']['posters'], urlbases, language)
        landscape = _get_images(movie['images']['backdrops'], urlbases, language)
        fanart = _get_images(movie['images']['backdrops'], urlbases, None)

    setposters = []
    setlandscape = []
    setfanart = []
    if collection and 'images' in collection:
        setposters = _get_images_with_fallback(collection['images']['posters'], urlbases, language)
        setlandscape = _get_images(collection['images']['backdrops'], urlbases, language)
        setfanart = _get_images(collection['images']['backdrops'], urlbases, None)

    return {'poster': posters, 'landscape': landscape, 'fanart': fanart,
        'set.poster': setposters, 'set.landscape': setlandscape, 'set.fanart': setfanart}

def _get_images_with_fallback(imagelist, urlbases, language, language_fallback='en'):
    images = _get_images(imagelist, urlbases, language)

    # Add backup images
    if language != language_fallback:
        images.extend(_get_images(imagelist, urlbases, language_fallback))

    # Add any images if nothing set so far
    if not images:
        images = _get_images(imagelist, urlbases)

    return images

def _get_images(imagelist, urlbases, language='_any'):
    result = []
    for img in imagelist:
        if language != '_any' and img['iso_639_1'] != language:
            continue
        result.append({
            'url': urlbases['original'] + img['file_path'],
            'preview': urlbases['preview'] + img['file_path'],
        })
    return result

def _get_date_numeric(datetime_):
    return (datetime_ - datetime(1970, 1, 1)).total_seconds()

def _load_base_urls(url_settings):
    urls = {}
    urls['original'] = url_settings.getSettingString('originalUrl')
    urls['preview'] = url_settings.getSettingString('previewUrl')
    last_updated = url_settings.getSettingString('lastUpdated')
    if not urls['original'] or not urls['preview'] or not last_updated or \
            float(last_updated) < _get_date_numeric(datetime.now() - timedelta(days=30)):
        conf = tmdbapi.get_configuration()
        if conf:
            urls['original'] = conf['images']['secure_base_url'] + 'original'
            urls['preview'] = conf['images']['secure_base_url'] + 'w780'
            url_settings.setSetting('originalUrl', urls['original'])
            url_settings.setSetting('previewUrl', urls['preview'])
            url_settings.setSetting('lastUpdated', str(_get_date_numeric(datetime.now())))
    return urls

def _parse_trailer(trailers, fallback):
    if trailers.get('youtube'):
        return 'plugin://plugin.video.youtube/?action=play_video&videoid='+trailers['youtube'][0]['source']
    if fallback.get('youtube'):
        return 'plugin://plugin.video.youtube/?action=play_video&videoid='+fallback['youtube'][0]['source']
    return None

def _get_names(items):
    return [item['name'] for item in items] if items else []

def _get_cast_members(casts, casttype, department, jobs):
    result = []
    if casttype in casts:
        for cast in casts[casttype]:
            if cast['department'] == department and cast['job'] in jobs and cast['name'] not in result:
                result.append(cast['name'])
    return result
